# -*- coding: utf-8 -*-

###########################################################################
## Python code generated with wxFormBuilder (version Jun  6 2014)
## http://www.wxformbuilder.org/
##
## PLEASE DO "NOT" EDIT THIS FILE!
###########################################################################

import wx
import wx.xrc
import wxx
import wx.richtext
import wx.propgrid as pg

###########################################################################
## Class FilePane
###########################################################################

class FilePane ( wx.Panel ):
	
	def __init__( self, parent ):
		wx.Panel.__init__ ( self, parent, id = wx.ID_ANY, pos = wx.DefaultPosition, size = wx.Size( 500,300 ), style = wx.TAB_TRAVERSAL|wx.WANTS_CHARS )
		
		self.SetExtraStyle( wx.WS_EX_BLOCK_EVENTS )
		
		fgSizer28 = wx.FlexGridSizer( 1, 1, 0, 0 )
		fgSizer28.AddGrowableCol( 0 )
		fgSizer28.AddGrowableRow( 0 )
		fgSizer28.SetFlexibleDirection( wx.BOTH )
		fgSizer28.SetNonFlexibleGrowMode( wx.FLEX_GROWMODE_SPECIFIED )
		
		import app.ui.ctrl.Editor as Editor
		self.m_editor = Editor(self)
		fgSizer28.Add( self.m_editor, 1, wx.EXPAND, 5 )
		
		
		self.SetSizer( fgSizer28 )
		self.Layout()
		
		# Connect Events
		self.m_editor.Bind( wx.EVT_KILL_FOCUS, self.OnKillFocus )
		self.m_editor.Bind( wx.EVT_SET_FOCUS, self.OnGetFocus )
	
	def __del__( self ):
		pass
	
	
	# Virtual event handlers, overide them in your derived class
	def OnKillFocus( self, event ):
		event.Skip()
	
	def OnGetFocus( self, event ):
		event.Skip()
	

###########################################################################
## Class TextPane
###########################################################################

class TextPane ( wx.Panel ):
	
	def __init__( self, parent ):
		wx.Panel.__init__ ( self, parent, id = wx.ID_ANY, pos = wx.DefaultPosition, size = wx.Size( 500,300 ), style = wx.TAB_TRAVERSAL|wx.WANTS_CHARS )
		
		self.SetExtraStyle( wx.WS_EX_BLOCK_EVENTS )
		
		fgSizer28 = wx.FlexGridSizer( 1, 1, 0, 0 )
		fgSizer28.AddGrowableCol( 0 )
		fgSizer28.AddGrowableRow( 0 )
		fgSizer28.SetFlexibleDirection( wx.BOTH )
		fgSizer28.SetNonFlexibleGrowMode( wx.FLEX_GROWMODE_SPECIFIED )
		
		import app.ui.ctrl.Editor as Editor
		self.m_editor = Editor(self)
		fgSizer28.Add( self.m_editor, 1, wx.EXPAND, 5 )
		
		
		self.SetSizer( fgSizer28 )
		self.Layout()
		
		# Connect Events
		self.m_editor.Bind( wx.EVT_KILL_FOCUS, self.OnKillFocus )
		self.m_editor.Bind( wx.EVT_SET_FOCUS, self.OnGetFocus )
	
	def __del__( self ):
		pass
	
	
	# Virtual event handlers, overide them in your derived class
	def OnKillFocus( self, event ):
		event.Skip()
	
	def OnGetFocus( self, event ):
		event.Skip()
	

###########################################################################
## Class FilePythonPane
###########################################################################

class FilePythonPane ( wx.Panel ):
	
	def __init__( self, parent ):
		wx.Panel.__init__ ( self, parent, id = wx.ID_ANY, pos = wx.DefaultPosition, size = wx.Size( 500,300 ), style = wx.TAB_TRAVERSAL|wx.WANTS_CHARS )
		
		self.SetExtraStyle( wx.WS_EX_BLOCK_EVENTS )
		
		fgSizer28 = wx.FlexGridSizer( 1, 1, 0, 0 )
		fgSizer28.AddGrowableCol( 0 )
		fgSizer28.AddGrowableRow( 0 )
		fgSizer28.SetFlexibleDirection( wx.BOTH )
		fgSizer28.SetNonFlexibleGrowMode( wx.FLEX_GROWMODE_SPECIFIED )
		
		import app.ui.ctrl.Editor as Editor
		self.m_editor = Editor(self, language='python')
		fgSizer28.Add( self.m_editor, 1, wx.EXPAND, 5 )
		
		
		self.SetSizer( fgSizer28 )
		self.Layout()
		
		# Connect Events
		self.m_editor.Bind( wx.EVT_KILL_FOCUS, self.OnKillFocus )
		self.m_editor.Bind( wx.EVT_SET_FOCUS, self.OnGetFocus )
	
	def __del__( self ):
		pass
	
	
	# Virtual event handlers, overide them in your derived class
	def OnKillFocus( self, event ):
		event.Skip()
	
	def OnGetFocus( self, event ):
		event.Skip()
	

###########################################################################
## Class FilesView
###########################################################################

class FilesView ( wx.Panel ):
	
	def __init__( self, parent ):
		wx.Panel.__init__ ( self, parent, id = wx.ID_ANY, pos = wx.DefaultPosition, size = wx.Size( 500,300 ), style = wx.TAB_TRAVERSAL|wx.WANTS_CHARS )
		
		fgSizer3 = wx.FlexGridSizer( 1, 1, 0, 0 )
		fgSizer3.AddGrowableCol( 0 )
		fgSizer3.AddGrowableRow( 0 )
		fgSizer3.SetFlexibleDirection( wx.BOTH )
		fgSizer3.SetNonFlexibleGrowMode( wx.FLEX_GROWMODE_SPECIFIED )
		
		self.m_tree = wxx.TreeCtrl( self, wx.ID_ANY, wx.DefaultPosition, wx.DefaultSize, wx.TR_HAS_BUTTONS|wx.TR_HIDE_ROOT|wx.TR_SINGLE|wx.NO_BORDER )
		fgSizer3.Add( self.m_tree, 0, wx.EXPAND, 5 )
		
		
		self.SetSizer( fgSizer3 )
		self.Layout()
		
		# Connect Events
		self.Bind( wx.EVT_KILL_FOCUS, self.OnKillFocus )
		self.Bind( wx.EVT_SET_FOCUS, self.OnSetFocus )
	
	def __del__( self ):
		pass
	
	
	# Virtual event handlers, overide them in your derived class
	def OnKillFocus( self, event ):
		event.Skip()
	
	def OnSetFocus( self, event ):
		event.Skip()
	

###########################################################################
## Class NewFile
###########################################################################

class NewFile ( wx.Dialog ):
	
	def __init__( self, parent ):
		wx.Dialog.__init__ ( self, parent, id = wx.ID_ANY, title = u"NewFile", pos = wx.DefaultPosition, size = wx.Size( 421,356 ), style = wx.DEFAULT_DIALOG_STYLE )
		
		self.SetSizeHintsSz( wx.DefaultSize, wx.DefaultSize )
		
		fgSizer125 = wx.FlexGridSizer( 3, 1, 0, 0 )
		fgSizer125.AddGrowableCol( 0 )
		fgSizer125.AddGrowableRow( 1 )
		fgSizer125.SetFlexibleDirection( wx.BOTH )
		fgSizer125.SetNonFlexibleGrowMode( wx.FLEX_GROWMODE_SPECIFIED )
		
		fgSizer126 = wx.FlexGridSizer( 1, 2, 0, 0 )
		fgSizer126.AddGrowableCol( 1 )
		fgSizer126.SetFlexibleDirection( wx.BOTH )
		fgSizer126.SetNonFlexibleGrowMode( wx.FLEX_GROWMODE_ALL )
		
		self.m_staticText85 = wx.StaticText( self, wx.ID_ANY, u"File name", wx.DefaultPosition, wx.DefaultSize, 0 )
		self.m_staticText85.Wrap( -1 )
		fgSizer126.Add( self.m_staticText85, 0, wx.ALL|wx.ALIGN_CENTER_VERTICAL, 5 )
		
		self.m_file = wx.TextCtrl( self, wx.ID_ANY, wx.EmptyString, wx.DefaultPosition, wx.DefaultSize, 0 )
		fgSizer126.Add( self.m_file, 0, wx.ALIGN_CENTER_VERTICAL|wx.EXPAND|wx.TOP|wx.RIGHT|wx.LEFT, 5 )
		
		
		fgSizer125.Add( fgSizer126, 0, wx.ALIGN_CENTER_VERTICAL|wx.ALIGN_CENTER_HORIZONTAL|wx.EXPAND|wx.ALL, 5 )
		
		sbSizer9 = wx.StaticBoxSizer( wx.StaticBox( self, wx.ID_ANY, u"Documentation" ), wx.VERTICAL )
		
		self.m_richText3 = wx.richtext.RichTextCtrl( self, wx.ID_ANY, wx.EmptyString, wx.DefaultPosition, wx.DefaultSize, 0|wx.HSCROLL|wx.SUNKEN_BORDER|wx.VSCROLL|wx.WANTS_CHARS )
		sbSizer9.Add( self.m_richText3, 1, wx.EXPAND|wx.ALL, 5 )
		
		
		fgSizer125.Add( sbSizer9, 1, wx.EXPAND, 5 )
		
		fgSizer158 = wx.FlexGridSizer( 1, 2, 0, 0 )
		fgSizer158.AddGrowableCol( 1 )
		fgSizer158.SetFlexibleDirection( wx.BOTH )
		fgSizer158.SetNonFlexibleGrowMode( wx.FLEX_GROWMODE_SPECIFIED )
		
		self.m_info = wx.BitmapButton( self, wx.ID_ANY, wx.ArtProvider.GetBitmap( wx.ART_TIP, wx.ART_BUTTON ), wx.DefaultPosition, wx.DefaultSize, wx.BU_AUTODRAW|wx.NO_BORDER )
		fgSizer158.Add( self.m_info, 0, wx.ALL, 5 )
		
		m_sdbSizer10 = wx.StdDialogButtonSizer()
		self.m_sdbSizer10OK = wx.Button( self, wx.ID_OK )
		m_sdbSizer10.AddButton( self.m_sdbSizer10OK )
		self.m_sdbSizer10Cancel = wx.Button( self, wx.ID_CANCEL )
		m_sdbSizer10.AddButton( self.m_sdbSizer10Cancel )
		m_sdbSizer10.Realize();
		
		fgSizer158.Add( m_sdbSizer10, 1, wx.EXPAND, 5 )
		
		
		fgSizer125.Add( fgSizer158, 1, wx.EXPAND, 5 )
		
		
		self.SetSizer( fgSizer125 )
		self.Layout()
		
		self.Centre( wx.BOTH )
		
		# Connect Events
		self.m_sdbSizer10Cancel.Bind( wx.EVT_BUTTON, self.OnCancel )
		self.m_sdbSizer10OK.Bind( wx.EVT_BUTTON, self.OnOK )
	
	def __del__( self ):
		pass
	
	
	# Virtual event handlers, overide them in your derived class
	def OnCancel( self, event ):
		event.Skip()
	
	def OnOK( self, event ):
		event.Skip()
	

###########################################################################
## Class NewDir
###########################################################################

class NewDir ( wx.Dialog ):
	
	def __init__( self, parent ):
		wx.Dialog.__init__ ( self, parent, id = wx.ID_ANY, title = u"New directory", pos = wx.DefaultPosition, size = wx.Size( 421,356 ), style = wx.DEFAULT_DIALOG_STYLE )
		
		self.SetSizeHintsSz( wx.DefaultSize, wx.DefaultSize )
		
		fgSizer6 = wx.FlexGridSizer( 3, 1, 0, 0 )
		fgSizer6.AddGrowableCol( 0 )
		fgSizer6.AddGrowableRow( 1 )
		fgSizer6.SetFlexibleDirection( wx.BOTH )
		fgSizer6.SetNonFlexibleGrowMode( wx.FLEX_GROWMODE_SPECIFIED )
		
		fgSizer7 = wx.FlexGridSizer( 1, 2, 0, 0 )
		fgSizer7.AddGrowableCol( 1 )
		fgSizer7.SetFlexibleDirection( wx.BOTH )
		fgSizer7.SetNonFlexibleGrowMode( wx.FLEX_GROWMODE_SPECIFIED )
		
		self.m_staticText4 = wx.StaticText( self, wx.ID_ANY, u"Name", wx.DefaultPosition, wx.DefaultSize, 0 )
		self.m_staticText4.Wrap( -1 )
		fgSizer7.Add( self.m_staticText4, 0, wx.ALL|wx.ALIGN_CENTER_VERTICAL|wx.ALIGN_RIGHT, 5 )
		
		self.m_dirPicker = wx.DirPickerCtrl( self, wx.ID_ANY, wx.EmptyString, u"Select a folder", wx.DefaultPosition, wx.DefaultSize, wx.DIRP_USE_TEXTCTRL )
		fgSizer7.Add( self.m_dirPicker, 0, wx.ALL|wx.EXPAND, 5 )
		
		
		fgSizer6.Add( fgSizer7, 0, wx.EXPAND|wx.ALL, 5 )
		
		sbSizer9 = wx.StaticBoxSizer( wx.StaticBox( self, wx.ID_ANY, u"Documentation" ), wx.VERTICAL )
		
		self.m_richText3 = wx.richtext.RichTextCtrl( self, wx.ID_ANY, wx.EmptyString, wx.DefaultPosition, wx.DefaultSize, 0|wx.HSCROLL|wx.SUNKEN_BORDER|wx.VSCROLL|wx.WANTS_CHARS )
		sbSizer9.Add( self.m_richText3, 1, wx.EXPAND|wx.ALL, 5 )
		
		
		fgSizer6.Add( sbSizer9, 1, wx.EXPAND, 5 )
		
		fgSizer162 = wx.FlexGridSizer( 1, 2, 0, 0 )
		fgSizer162.AddGrowableCol( 1 )
		fgSizer162.SetFlexibleDirection( wx.BOTH )
		fgSizer162.SetNonFlexibleGrowMode( wx.FLEX_GROWMODE_SPECIFIED )
		
		self.m_info = wx.BitmapButton( self, wx.ID_ANY, wx.ArtProvider.GetBitmap( wx.ART_TIP, wx.ART_BUTTON ), wx.DefaultPosition, wx.DefaultSize, wx.BU_AUTODRAW|wx.NO_BORDER )
		fgSizer162.Add( self.m_info, 0, wx.ALL, 5 )
		
		m_sdbSizer2 = wx.StdDialogButtonSizer()
		self.m_sdbSizer2OK = wx.Button( self, wx.ID_OK )
		m_sdbSizer2.AddButton( self.m_sdbSizer2OK )
		self.m_sdbSizer2Cancel = wx.Button( self, wx.ID_CANCEL )
		m_sdbSizer2.AddButton( self.m_sdbSizer2Cancel )
		m_sdbSizer2.Realize();
		
		fgSizer162.Add( m_sdbSizer2, 1, wx.EXPAND|wx.TOP|wx.BOTTOM, 5 )
		
		
		fgSizer6.Add( fgSizer162, 1, wx.EXPAND, 5 )
		
		
		self.SetSizer( fgSizer6 )
		self.Layout()
		
		self.Centre( wx.BOTH )
		
		# Connect Events
		self.m_sdbSizer2Cancel.Bind( wx.EVT_BUTTON, self.OnCancel )
		self.m_sdbSizer2OK.Bind( wx.EVT_BUTTON, self.OnOK )
	
	def __del__( self ):
		pass
	
	
	# Virtual event handlers, overide them in your derived class
	def OnCancel( self, event ):
		event.Skip()
	
	def OnOK( self, event ):
		event.Skip()
	

###########################################################################
## Class Threads
###########################################################################

class Threads ( wx.Panel ):
	
	def __init__( self, parent ):
		wx.Panel.__init__ ( self, parent, id = wx.ID_ANY, pos = wx.DefaultPosition, size = wx.Size( 468,421 ), style = wx.TAB_TRAVERSAL )
		
		fgSizer10 = wx.FlexGridSizer( 1, 1, 0, 0 )
		fgSizer10.AddGrowableCol( 0 )
		fgSizer10.AddGrowableRow( 0 )
		fgSizer10.SetFlexibleDirection( wx.BOTH )
		fgSizer10.SetNonFlexibleGrowMode( wx.FLEX_GROWMODE_SPECIFIED )
		
		self.m_panel1 = wx.Panel( self, wx.ID_ANY, wx.DefaultPosition, wx.DefaultSize, wx.TAB_TRAVERSAL )
		fgSizer11 = wx.FlexGridSizer( 1, 1, 0, 0 )
		fgSizer11.AddGrowableCol( 0 )
		fgSizer11.AddGrowableRow( 0 )
		fgSizer11.SetFlexibleDirection( wx.BOTH )
		fgSizer11.SetNonFlexibleGrowMode( wx.FLEX_GROWMODE_SPECIFIED )
		
		m_listThreadsChoices = []
		self.m_listThreads = wx.ListBox( self.m_panel1, wx.ID_ANY, wx.DefaultPosition, wx.DefaultSize, m_listThreadsChoices, 0 )
		fgSizer11.Add( self.m_listThreads, 1, wx.EXPAND, 5 )
		
		
		self.m_panel1.SetSizer( fgSizer11 )
		self.m_panel1.Layout()
		fgSizer11.Fit( self.m_panel1 )
		fgSizer10.Add( self.m_panel1, 1, wx.EXPAND, 5 )
		
		
		self.SetSizer( fgSizer10 )
		self.Layout()
		
		# Connect Events
		self.m_listThreads.Bind( wx.EVT_LISTBOX, self.OnSelectThread )
		self.m_listThreads.Bind( wx.EVT_LISTBOX_DCLICK, self.OnActivateThread )
	
	def __del__( self ):
		pass
	
	
	# Virtual event handlers, overide them in your derived class
	def OnSelectThread( self, event ):
		event.Skip()
	
	def OnActivateThread( self, event ):
		event.Skip()
	

###########################################################################
## Class StackFrame
###########################################################################

class StackFrame ( wx.Panel ):
	
	def __init__( self, parent ):
		wx.Panel.__init__ ( self, parent, id = wx.ID_ANY, pos = wx.DefaultPosition, size = wx.Size( 500,300 ), style = wx.TAB_TRAVERSAL )
		
		fgSizer12 = wx.FlexGridSizer( 1, 1, 0, 0 )
		fgSizer12.AddGrowableCol( 0 )
		fgSizer12.AddGrowableRow( 0 )
		fgSizer12.SetFlexibleDirection( wx.BOTH )
		fgSizer12.SetNonFlexibleGrowMode( wx.FLEX_GROWMODE_SPECIFIED )
		
		m_stackFrameChoices = []
		self.m_stackFrame = wx.ListBox( self, wx.ID_ANY, wx.DefaultPosition, wx.DefaultSize, m_stackFrameChoices, wx.LB_SINGLE )
		fgSizer12.Add( self.m_stackFrame, 1, wx.EXPAND, 5 )
		
		
		self.SetSizer( fgSizer12 )
		self.Layout()
		
		# Connect Events
		self.m_stackFrame.Bind( wx.EVT_LISTBOX, self.OnSelectStackFrame )
		self.m_stackFrame.Bind( wx.EVT_LISTBOX_DCLICK, self.OnActivateStackFrame )
	
	def __del__( self ):
		pass
	
	
	# Virtual event handlers, overide them in your derived class
	def OnSelectStackFrame( self, event ):
		event.Skip()
	
	def OnActivateStackFrame( self, event ):
		event.Skip()
	

###########################################################################
## Class Locals
###########################################################################

class Locals ( wx.Panel ):
	
	def __init__( self, parent ):
		wx.Panel.__init__ ( self, parent, id = wx.ID_ANY, pos = wx.DefaultPosition, size = wx.Size( 500,300 ), style = wx.TAB_TRAVERSAL )
		
		fgSizer13 = wx.FlexGridSizer( 1, 1, 0, 0 )
		fgSizer13.AddGrowableCol( 0 )
		fgSizer13.AddGrowableRow( 0 )
		fgSizer13.SetFlexibleDirection( wx.BOTH )
		fgSizer13.SetNonFlexibleGrowMode( wx.FLEX_GROWMODE_SPECIFIED )
		
		self.m_locals = pg.PropertyGrid(self, wx.ID_ANY, wx.DefaultPosition, wx.DefaultSize, wx.propgrid.PG_DEFAULT_STYLE)
		fgSizer13.Add( self.m_locals, 0, wx.EXPAND, 5 )
		
		
		self.SetSizer( fgSizer13 )
		self.Layout()
	
	def __del__( self ):
		pass
	

###########################################################################
## Class Expressions
###########################################################################

class Expressions ( wx.Panel ):
	
	def __init__( self, parent ):
		wx.Panel.__init__ ( self, parent, id = wx.ID_ANY, pos = wx.DefaultPosition, size = wx.Size( 500,300 ), style = wx.TAB_TRAVERSAL )
		
		fgSizer13 = wx.FlexGridSizer( 1, 1, 0, 0 )
		fgSizer13.AddGrowableCol( 0 )
		fgSizer13.AddGrowableRow( 0 )
		fgSizer13.SetFlexibleDirection( wx.BOTH )
		fgSizer13.SetNonFlexibleGrowMode( wx.FLEX_GROWMODE_SPECIFIED )
		
		self.m_propertyGrid1 = pg.PropertyGrid(self, wx.ID_ANY, wx.DefaultPosition, wx.DefaultSize, wx.propgrid.PG_DEFAULT_STYLE)
		fgSizer13.Add( self.m_propertyGrid1, 0, wx.EXPAND, 5 )
		
		
		self.SetSizer( fgSizer13 )
		self.Layout()
	
	def __del__( self ):
		pass
	

###########################################################################
## Class Breakpoints
###########################################################################

class Breakpoints ( wx.Panel ):
	
	def __init__( self, parent ):
		wx.Panel.__init__ ( self, parent, id = wx.ID_ANY, pos = wx.DefaultPosition, size = wx.Size( 459,479 ), style = wx.TAB_TRAVERSAL )
		
		fgSizer13 = wx.FlexGridSizer( 1, 1, 0, 0 )
		fgSizer13.AddGrowableCol( 0 )
		fgSizer13.AddGrowableRow( 0 )
		fgSizer13.SetFlexibleDirection( wx.BOTH )
		fgSizer13.SetNonFlexibleGrowMode( wx.FLEX_GROWMODE_SPECIFIED )
		
		self.m_listBreakpoints = wx.ListCtrl( self, wx.ID_ANY, wx.DefaultPosition, wx.DefaultSize, wx.LC_NO_HEADER|wx.LC_REPORT|wx.LC_SINGLE_SEL )
		fgSizer13.Add( self.m_listBreakpoints, 1, wx.EXPAND, 5 )
		
		
		self.SetSizer( fgSizer13 )
		self.Layout()
		
		# Connect Events
		self.m_listBreakpoints.Bind( wx.EVT_LIST_COL_RIGHT_CLICK, self.OnBreakpoinMenu )
		self.m_listBreakpoints.Bind( wx.EVT_LIST_ITEM_ACTIVATED, self.OnEditBreakpoint )
		self.m_listBreakpoints.Bind( wx.EVT_LIST_ITEM_SELECTED, self.OnSelectBreakpoint )
		self.m_listBreakpoints.Bind( wx.EVT_SIZE, self.OnSize )
	
	def __del__( self ):
		pass
	
	
	# Virtual event handlers, overide them in your derived class
	def OnBreakpoinMenu( self, event ):
		event.Skip()
	
	def OnEditBreakpoint( self, event ):
		event.Skip()
	
	def OnSelectBreakpoint( self, event ):
		event.Skip()
	
	def OnSize( self, event ):
		event.Skip()
	

###########################################################################
## Class DebugCommand
###########################################################################

class DebugCommand ( wx.Panel ):
	
	def __init__( self, parent ):
		wx.Panel.__init__ ( self, parent, id = wx.ID_ANY, pos = wx.DefaultPosition, size = wx.Size( 500,300 ), style = wx.TAB_TRAVERSAL )
		
		fgSizer18 = wx.FlexGridSizer( 2, 1, 0, 0 )
		fgSizer18.AddGrowableCol( 0 )
		fgSizer18.AddGrowableRow( 0 )
		fgSizer18.SetFlexibleDirection( wx.BOTH )
		fgSizer18.SetNonFlexibleGrowMode( wx.FLEX_GROWMODE_SPECIFIED )
		
		self.m_debugOutput = wx.TextCtrl( self, wx.ID_ANY, wx.EmptyString, wx.DefaultPosition, wx.DefaultSize, wx.TE_MULTILINE|wx.TE_READONLY )
		fgSizer18.Add( self.m_debugOutput, 0, wx.EXPAND, 5 )
		
		fgSizer19 = wx.FlexGridSizer( 1, 2, 0, 0 )
		fgSizer19.AddGrowableCol( 1 )
		fgSizer19.AddGrowableRow( 0 )
		fgSizer19.SetFlexibleDirection( wx.BOTH )
		fgSizer19.SetNonFlexibleGrowMode( wx.FLEX_GROWMODE_SPECIFIED )
		
		self.m_staticText3 = wx.StaticText( self, wx.ID_ANY, u"command:", wx.DefaultPosition, wx.DefaultSize, 0 )
		self.m_staticText3.Wrap( -1 )
		fgSizer19.Add( self.m_staticText3, 0, wx.ALL, 5 )
		
		self.m_debugCommand = wx.TextCtrl( self, wx.ID_ANY, wx.EmptyString, wx.DefaultPosition, wx.DefaultSize, wx.TE_PROCESS_ENTER )
		fgSizer19.Add( self.m_debugCommand, 0, wx.EXPAND, 5 )
		
		
		fgSizer18.Add( fgSizer19, 1, wx.EXPAND, 5 )
		
		
		self.SetSizer( fgSizer18 )
		self.Layout()
		
		# Connect Events
		self.m_debugCommand.Bind( wx.EVT_TEXT_ENTER, self.OnEnter )
	
	def __del__( self ):
		pass
	
	
	# Virtual event handlers, overide them in your derived class
	def OnEnter( self, event ):
		event.Skip()
	




"""Subclass of NewDestructor, which is generated by wxFormBuilder."""

import wx
import wxx
from activity.models.ui import ui as ui
import app.resources as rc


# Implementing NewDestructor
class DestructorDialog(ui.NewDestructor):
    """
    This dialog allows the edition of class or struct destructor.
    """
    @wxx.SetInfo(__doc__)
    def __init__(self, parent, container):
        """Initialization"""
        super(DestructorDialog, self).__init__(parent)
        self._container = container
        self._class = container.inner_class
        self.m_choice2.SetFocus()
        icon = wx.EmptyIcon()
        icon.CopyFromBitmap(rc.GetBitmap("destructor"))
        self.SetIcon(icon)

    def CopyAttributes(self, dtor):
        """Transfer attributes to dtor"""
        dtor._name = self._name
        dtor._access = self._access
        dtor._inline = self._inline
        dtor._virtual = self._virtual
        dtor._pure = self._pure
        dtor._declare = self._declare
        dtor._implement = self._implement
        dtor._calling = self._calling
        if self._calling:
            dtor._callingText = self._callingText
        dtor.note = self._note

    def SetAttributes(self, dtor):
        """Transfer attributes from existing constructor"""
        self._container = dtor.parent
        self._class = dtor.inner_class
        self._name = dtor._name
        self._access = dtor._access
        iSel = self.m_choice2.FindString(self._access)
        self.m_choice2.Select(iSel)
        self._inline = dtor._inline
        self.m_checkBox31.SetValue(self._inline)
        self._virtual = dtor._virtual
        self.m_checkBox41.SetValue(self._virtual)
        self._pure = dtor._pure
        self.m_checkBox104.SetValue(self._pure)
        self._declare = dtor._declare
        self.m_checkBox92.SetValue(self._declare)
        self._implement = dtor._implement
        self.m_checkBox93.SetValue(self._implement)
        self._calling = dtor._calling
        self.m_checkBox6.SetValue(self._calling)
        self.m_comboBox1.Enable(self._calling)
        if self._calling:
            self._callingText = dtor._callingText
            iSel = self.m_comboBox1.FindString(self._callingText)
            self.m_comboBox1.Select(iSel)
        self._note = dtor.note
        self.m_richText1.SetValue(self._note)
        self.SetTitle("Edit destructor")

    def Validate(self):
        """Validation"""
        self._name = self._class._name
        iSel = self.m_choice2.GetSelection()
        self._access = self.m_choice2.GetString(iSel)
        self._inline = self.m_checkBox31.IsChecked()
        self._virtual = self.m_checkBox41.IsChecked()
        self._pure = self.m_checkBox104.IsChecked()
        self._declare = self.m_checkBox92.IsChecked()
        self._implement = self.m_checkBox93.IsChecked()
        self._calling = self.m_checkBox6.IsChecked()
        if self._calling:
            iSel = self.m_comboBox1.GetSelection()
            self._callingText = self.m_comboBox1.GetString(iSel)
        self._note = self.m_richText1.GetValue()
        return True

    def get_kwargs(self):
        """Returns the kwargs needed for this object"""
        kwargs = {}
        kwargs['parent'] = self._container
        kwargs['name'] = self._name
        kwargs['access'] = self._access
        kwargs['inline'] = self._inline
        kwargs['virtual'] = self._virtual
        kwargs['pure'] = self._pure
        kwargs['declare'] = self._declare
        kwargs['implement'] = self._implement
        kwargs['calling'] = self._calling
        kwargs['callspec'] = (self._calling and self._callingText) or ''
        kwargs['note'] = self._note
        return kwargs

    def OnCancel(self, event):
        """Implement OnCance"""
        self.EndModal(wx.ID_CANCEL)

    def OnOK(self, event):
        """Implement OnOk"""
        if self.Validate():
            self.EndModal(wx.ID_OK)





# -*- coding: utf-8 -*-

import mimetypes
import wx
import app.resources as rc
import deco


class MimeHandler(object):
    """Class for handling mime types"""
    instance = None

    def __init__(self):
        """init"""
        import gtk
        import gio
        self._extra_bitmap = {}
        self._handler = wx.MimeTypesManager()
        self._mime = self._handler.EnumAllFileTypes()
        self._mime_themed_icon = dict([(x, gio.content_type_get_icon(x)) for x in self._mime])
        self._themed_icon = []
        for themed_icon in self._mime_themed_icon.values():
            if themed_icon not in self._themed_icon:
                self._themed_icon.append(themed_icon)
        self._theme = gtk.icon_theme_get_default()
        self._size = 24
        self._imageList = None
        self.register_default_extra_bitmaps()
        MimeHandler.instance = self
        super(MimeHandler, self).__init__()

    @deco.classproperty
    def imageList(cls):
        """returns the image list"""
        if not MimeHandler.instance:
            MimeHandler.instance = MimeHandler()
        if not MimeHandler.instance._imageList:
            MimeHandler.instance.realize()
        return MimeHandler.instance._imageList

    def realize(self):
        """ends the construction of the mime handler"""
        self._icons = self._load_icons()
        self._themed_icon_index = self._map_icon_index()
        self._imageList = self._create_image_list()

    def register_default_extra_bitmaps(self):
        """Adds some extra bitmaps that will be included
        in the image list"""
        self._extra_bitmap.update(
            dict([
                (i, wx.BitmapFromXPMData(rc._xpm[i]))
                for i in range(0, len(rc._xpm))]))

    def register_extra_bitmap(self, position, bitmap):
        """Some extra bitmaps may be added to the image list
        at specific positions"""
        self._extra_bitmap[position] = bitmap

    def _load_icons(self):
        """Reload icons for some size and returns a dict"""
        icons = {}
        size = self._size
        for themed_icon in self._themed_icon:
            info = self._theme.choose_icon(themed_icon.get_names(), size, 0)
            if info:
                icons[themed_icon] = wx.Icon(info.get_filename(), desiredWidth=size)
            else:
                icons[themed_icon] = None
        return icons

    def _map_icon_index(self):
        """Create a map for the themed icon and image index"""
        index = 0
        themed_icon_index = {}
        for themed_icon in self._themed_icon:
            if self._icons[themed_icon]:
                # skip registered bitmaps that have fixed positions
                while index in self._extra_bitmap:
                    index = index + 1
                themed_icon_index[themed_icon] = index
                index = index + 1
            else:
                themed_icon_index[themed_icon] = wx.NOT_FOUND
        return themed_icon_index

    def _create_image_list(self):
        """Create an image list based on current icons"""
        index = 0
        icons = [x for x in self._icons.values() if x]
        iml = wx.ImageList(self._size, self._size, True, len(icons))
        for themed_icon in self._themed_icon:
            ico = self._icons[themed_icon]
            if ico is None:
                continue
            while index in self._extra_bitmap:
                iml.Add(self._extra_bitmap[index])
                index = index + 1
            if ico.GetWidth() != self._size or ico.GetHeight() != self._size:
                img = wx.BitmapFromIcon(ico).ConvertToImage()
                sz = wx.Size(self._size, self._size)
                if ico.GetWidth() >= ico.GetHeight():
                    fact = float(self._size) / float(ico.GetWidth())
                    h = int(float(ico.GetHeight()) * fact)
                    img = img.Rescale(self._size, h, wx.IMAGE_QUALITY_NORMAL)
                    pos = wx.Point(0, int((self._size - h) / 2))
                    img = img.Resize(sz, pos)
                else:
                    fact = float(self._size) / float(ico.GetHeight())
                    w = int(float(ico.GetWidth()) * fact)
                    img = img.Rescale(w, self._size, wx.IMAGE_QUALITY_NORMAL)
                    pos = wx.Point(int((self._size - w) / 2), 0)
                    img = img.Resize(sz, pos)
                bmp = img.ConvertToBitmap()
                iml.Add(bmp)
            else:
                iml.AddIcon(ico)
            index = index + 1
        return iml

    @classmethod
    def file_image_index(cls, path):
        """Returns the associated image index or wx.NOT_FOUND, based
        on mime type"""
        self = cls.instance
        use_gtk = True  # gtk mime-resolution is much better than extension-based
        if use_gtk:
            import gio
            f = gio.File(path)
            info = f.query_info('standard::content-type')
            mime = info.get_content_type()
        else:
            mime = mimetypes.guess_type(path)[0]
        if not mime:
            return wx.NOT_FOUND
        if mime not in self._mime:
            return wx.NOT_FOUND
        # ok, we can go from mime to themed icon and from that to icon index
        themed = self._mime_themed_icon[mime]
        return self._themed_icon_index[themed]



"""Subclass of NewProject, which is generated by wxFormBuilder."""

import os, wx, wxx, app
import app.resources as rc
from ctx import localcontext as context


# Implementing NewProject
class ProjectDialog(app.NewProject):
    """
    This dialog is the start point for a new project to
    be created. You  may select the target language and
    set his common options. The language may be set only
    before the project is created.
    """
    @wxx.SetInfo(__doc__)
    def __init__(self, parent, container=None):
        """Initialization"""
        super(ProjectDialog, self).__init__(parent)
        stdPaths = wx.StandardPaths.Get()
        self._container = container
        # if this project is inside workspace, the base
        # path must be determined by this workspace
        wk = container.workspace
        if wk is not None:
            self._base = wk._dir
            self.m_bpButton3.Enable(False)
            self.m_textCtrl9.SetEditable(False)
        else:
            self._base = stdPaths.GetDocumentsDir()
        self.m_textCtrl9.SetValue(self._base)
        self.app = context.app
        self._original = ""
        self._autodir = True
        self._veto = False
        self.m_textCtrl1.SetFocus()
        icon = wx.EmptyIcon()
        icon.CopyFromBitmap(rc.GetBitmap("project"))
        self.SetIcon(icon)

    def CopyAttributes(self, prj):
        """Update project attributes with dialog values"""
        sel = self.m_choicebook1.GetSelection()
        if sel == 0:
            prj._language = 'c++'
        if sel == 1:
            prj._language = 'python'
        prj._name = self._name
        prj._dir = self._dir
        prj._includeDir = self.m_textCtrl23.GetValue()
        prj._srcDir = self.m_textCtrl24.GetValue()
        prj._useMaster = self.m_checkBox52.GetValue()
        prj._masterInclude = self.m_textCtrl25.GetValue()
        prj._author = self.m_textCtrl251.GetValue()
        prj._date = self.m_datePicker1.GetValue().FormatISODate()
        if self.m_checkBox53.IsChecked():
            prj._license = self.m_choice16.GetStringSelection()
        else:
            prj._license = None
        prj._type = self.m_choice151.GetStringSelection()
        prj._version[0] = self.m_spinCtrl11.GetValue()
        prj._version[1] = self.m_spinCtrl21.GetValue()
        prj._version[2] = self.m_spinCtrl31.GetValue()
        prj._useMakefile = self.m_checkBox54.IsChecked()
        prj.note = self.m_richText13.GetValue()

    def SetAttributes(self, prj):
        """Set dialog attributes based on existing project"""
        if prj._language == 'c++':
            self.m_choicebook1.SetSelection(0)
        if prj._language == 'python':
            self.m_choicebook1.SetSelection(1)
        self._veto = True
        self.m_textCtrl1.SetValue(prj._name)
        self.m_textCtrl9.SetValue(prj._dir)
        self.m_textCtrl23.SetValue(prj._includeDir)
        self.m_textCtrl24.SetValue(prj._srcDir)
        self.m_checkBox52.SetValue(prj._useMaster)
        self.m_textCtrl25.Enable(prj._useMaster)
        self.m_textCtrl25.SetValue(prj._masterInclude)
        self.m_textCtrl251.SetValue(prj._author)
        date = wx.DateTime()
        date.ParseDate(prj._date)
        self.m_datePicker1.SetValue(date)
        if prj._license is not None:
            self.m_checkBox53.SetValue(True)
            self.m_choice16.SetStringSelection(prj._license)
        else:
            self.m_checkBox53.SetValue(False)
        self.m_choice151.SetStringSelection(prj._type)
        self.m_spinCtrl11.SetValue(prj._version[0])
        self.m_spinCtrl21.SetValue(prj._version[1])
        self.m_spinCtrl31.SetValue(prj._version[2])
        self.m_checkBox54.SetValue(prj._useMakefile)
        self.m_richText13.SetValue(prj.note)

        self._original = prj._name
        self.SetTitle("Edit project")

    def OnPageChanging(self, event):
        """Checks if language can be changed"""
        if self._veto:
            event.Veto()
        else:
            event.Skip()

    def OnChangeProjectName(self, event):
        """Process change text event"""
        if self._autodir:
            name = self.m_textCtrl1.GetValue()
            #create a custom subdirectory
            self.m_textCtrl9.SetValue(os.path.join(self._base, name))
            if self.m_checkBox52.GetValue():
                #replace master include
                name = name.replace(" ", "_")
                name += ".h"
                self.m_textCtrl25.SetValue(name)

    def OnChooseDir(self, event):
        """Launch subdialog for choosing dir"""
        dialog = wx.DirDialog(self, "Select directory",
            self.m_textCtrl9.GetValue(), wx.DD_DIR_MUST_EXIST)
        if dialog.ShowModal() == wx.ID_OK:
            self.m_textCtrl9.SetValue(dialog.GetPath())
            self._autodir = False

    def OnToggleMasterInclude(self, event):
        """Toggle using master include file"""
        if self.m_checkBox52.GetValue():
            self.m_textCtrl25.Enable(True)
            self.m_textCtrl25.SetValue(self.m_textCtrl1.GetValue() + ".h")
        else:
            self.m_textCtrl25.Enable(False)

    def Validate(self):
        """Validate dialog"""
        sel = self.m_choicebook1.GetSelection()
        if sel == 0:
            self._language = 'c++'
        if sel == 1:
            self._language = 'python'
        self._name = self.m_textCtrl1.GetValue()
        self._dir = self.m_textCtrl9.GetValue()
        if len(self._name) == 0:
            wx.MessageBox("Project name must be non empty", "Error",
                wx.OK | wx.CENTER | wx.ICON_ERROR, self)
            return False
        if not self._original == self._name:
            if self.app.ExistProject(self._name) is True:
                wx.MessageBox("Project already exist", "Error",
                    wx.OK | wx.CENTER | wx.ICON_ERROR, self)
                return False
        if len(self._dir) == 0:
            wx.MessageBox("Target directory must be non empty",
                "Error", wx.OK | wx.CENTER | wx.ICON_ERROR, self)
            return False
        if self.m_checkBox52.GetValue():
            if len(self.m_textCtrl25.GetValue()) == 0:
                wx.MessageBox("Master include must be non empty",
                    "Error", wx.OK | wx.CENTER | wx.ICON_ERROR, self)
                return False
        self._includeDir = self.m_textCtrl23.GetValue()
        self._srcDir = self.m_textCtrl24.GetValue()
        self._useMaster = self.m_checkBox52.GetValue()
        self._masterInclude = self.m_textCtrl25.GetValue()
        self._author = self.m_textCtrl251.GetValue()
        self._date = self.m_datePicker1.GetValue().FormatISODate()
        self._license = self.m_choice16.GetStringSelection()
        self._type = self.m_choice151.GetStringSelection()
        self._version = []
        self._version = [
            self.m_spinCtrl11.GetValue(),
            self.m_spinCtrl21.GetValue(),
            self.m_spinCtrl31.GetValue()]
        self._useMakefile = True
        return True

    def get_kwargs(self):
        """returns arguments for instance... pending"""
        kwargs = {
            'parent': self._container,
            'language': self._language,
            'name': self._name,
            'dir': self._dir,
            'includedir': self._includeDir,
            'sourcedir': self._srcDir,
            'usemaster': self._useMaster,
            'masterinclude': self._masterInclude,
            'author': self._author,
            'date': self._date,
            'license': self._license,
            'type': self._type,
            'version': self._version,
            'usemakefile': self._useMakefile
        }
        return kwargs

    # Handlers for NewProject events.
    def OnOK(self, event):
        """Handle ok button"""
        if self.Validate():
            self.EndModal(wx.ID_OK)





# -*- coding: utf-8 -*-

import copy
import app.resources as rc
from ctx import theContext as context
import model.TComponent


class Workspace(model.TComponent):
    """Representation of Workspace"""

    #class_container = False
    #folder_container = False
    #diagram_container = True
    #module_container = False
    #namespace_container = False
    project_container = True
    repository_container = True

    def __init__(self, **kwargs):
        """Initialization of workspace"""
        self._dir = kwargs['dir']
        self._author = kwargs.get('author', "<unknown>")
        self._date = kwargs.get('date', "08-10-2966")
        self._license = kwargs.get('license', None)
        self._modified = True
        self._lastHdrTime = None
        super(Workspace, self).__init__(**kwargs)

    def OnUndoRedoRemoving(self, root=True):
        """Prepare for delete"""
        super(Workspace, self).OnUndoRedoRemoving(root)
        #context.app.RemoveWorkspace(self)

    def OnUndoRedoAdd(self):
        """Restore object from undo"""
        context.app.AddWorkspace(self)
        super(Workspace, self).OnUndoRedoAdd()

    def GetTabBitmap(self):
        """Get the bitmap for tab control"""
        return rc.GetBitmap('workspace')

    @property
    def bitmap_index(self):
        """Index of tree image"""
        return rc.GetBitmapIndex('workspace')

    def __getstate__(self):
        """Set pickle context"""
        # we dont want to serialize projects directly
        state = copy.copy(self.__dict__)
        state['_child'] = copy.copy(self._child)
        state['_child'][model.Project] = [
            (x._dir.replace("//", "/") + "/" + x._name + ".pcc").replace("//", "/")
            for x in self[model.Project] if type(x) is model.Project]
        return state

    def __setstate__(self, d):
        """Load pickle context"""
        plist = d['_child'].get(model.Project, None)
        if plist:
            del d['_child'][model.Project]
        self.__dict__ = d
        if plist:
            for pf in plist:
                context.app.LoadProject(pf, self)

    def SetModified(self, bModified=True):
        """Set the modified state"""
        self._lastHdrTime = None
        self._modified = bModified

    def IsModified(self):
        """Get the modified state"""
        return self._modified


# -*- coding: utf-8 -*-
"""
Created on Sun Dec 22 22:08:46 2013

@author: mel
"""
import app.resources as rc
import model.decorator as ctx
import tran
from MemberMethod import MemberMethod


class InitMethod(MemberMethod):
    """Implements member method"""

    # visual methods
    @tran.TransactionalMethod('move init method {}')
    def drop(self, to):
        """Drops datamember inside project or another folder """
        target = to.inner_member_container
        if not target or self.inner_class != target.inner_class or self.project != target.project:
            return False  # avoid move classes between projects
        index = 0
        tran.TransactionalMoveObject(
            object=self, origin=self.parent, target=target, index=index)
        return True

    def __init__(self, **kwargs):
        """Initialization"""
        kwargs["static_method"] = False
        kwargs["class_method"] = False
        kwargs["property"] = False
        kwargs["name"] = '__init__'
        super(InitMethod, self).__init__(**kwargs)

    def get_kwargs(self):
        """Returns the kwargs needed for this object"""
        kwargs = {}
        kwargs.update(super(InitMethod, self).get_kwargs())
        del kwargs['static_method']
        del kwargs['class_method']
        del kwargs['property']
        return kwargs

    @ctx.ContextDeclaration()
    def WriteDeclaration(self, pf):
        """Write the member method declaration"""
        pass

    @ctx.ContextImplementation()
    def WriteCode(self, f):
        """Write code to file"""
        pass

    def OnUndoRedoChanged(self):
        """Update from app"""
        super(InitMethod, self).OnUndoRedoChanged()

    def OnUndoRedoRemoving(self, root=True):
        """Prepare for delete"""
        super(InitMethod, self).OnUndoRedoRemoving(root)

    def OnUndoRedoAdd(self):
        """Restore object from undo"""
        super(InitMethod, self).OnUndoRedoAdd()

    def GetTabBitmap(self):
        """Get the bitmap for tab control"""
        return rc.GetBitmap("py_init")

    @property
    def bitmap_index(self):
        """Index of tree image"""
        return rc.GetBitmapIndex("py_init")

    @property
    def tree_label(self):
        """Get tree label"""
        from Argument import Argument
        from ArgsArgument import ArgsArgument
        from KwArgsArgument import KwArgsArgument
        alist = ', '.join(arg.label for arg in
            self[Argument] + self[ArgsArgument] + self[KwArgsArgument])
        return '__init__({arglist})'.format(arglist=alist)

    def ExistArgumentNamed(self, name):
        """Check about the existence of an argument"""
        from Argument import Argument
        from ArgsArgument import ArgsArgument
        from KwArgsArgument import KwArgsArgument
        return name in [x._name for x in self(Argument, ArgsArgument, KwArgsArgument)]




import wx


def JoinBitmap(leftBitmap, rightBitmap):
    """Create a new bitmap by joining"""
    width = leftBitmap.GetWidth() + rightBitmap.GetWidth()
    height = max(leftBitmap.GetHeight(), rightBitmap.GetHeight())
    leftOffset = (height - leftBitmap.GetHeight()) // 2
    rightOffset = (height - rightBitmap.GetHeight()) // 2
    image = wx.EmptyImage()
    image.Create(width, height, clear=True)
    image.Paste(leftBitmap.ConvertToImage(), 0, leftOffset)
    image.Paste(rightBitmap.ConvertToImage(),
        leftBitmap.GetWidth(), rightOffset)
    image.SetMaskColour(0, 0, 0)
    return image.ConvertToBitmap()

# Litte tool for debug
#
#counter=0
#def dumpBitmap(bmp):
#    global counter
#    name = '/home/mel/pythonProjects/proCxx/debug/bmp_{}.bmp'.format(counter)
#    bmp.SaveFile(name, wx.BITMAP_TYPE_BMP)
#    counter = counter + 1


def PrefixBitmapList(prefixBitmap, bitmapList, bitmapResult):
    """
    Prefixes a list of bitmaps with another bitmap
    and increases the original list.
    A map original_index:new_index is created and returned
    where original_index is the index in bitmapResult
    bitmapList must be a subset of bitmapResult
    """
    firstMap = {}
    newmap = {}
    i = 0
    j = len(bitmapResult)
    for bitmap in bitmapList:
        prefixed_bitmap = JoinBitmap(prefixBitmap, bitmap)
        bitmapResult.append(prefixed_bitmap)
        newmap[i] = j
        i = i + 1
        j = j + 1
    return newmap
    for bitmap in firstMap:
        newmap[bitmapList.index(bitmap)] = bitmapResult.index(
            firstMap[bitmap])
    return newmap

def SuffixBitmapList(suffixBitmap, bitmapList, bitmapResult):
    """
    Suffixes a list of bitmaps with another bitmap
    and increases the original list.
    A map original_index:new_index is created and returned
    where original_index is the index in bitmapResult
    bitmapList must be a subset of bitmapResult
    """
    firstMap = {}
    newmap = {}
    i = 0
    j = len(bitmapResult)
    for bitmap in bitmapList:
        suffixed_bitmap = JoinBitmap(bitmap, suffixBitmap)
        bitmapResult.append(suffixed_bitmap)
        newmap[i] = j
        i = i + 1
        j = j + 1
    return newmap
    for bitmap in firstMap:
        newmap[bitmapList.index(bitmap)] = bitmapResult.index(
            firstMap[bitmap])
    return newmap


def GetBitmapMaxSize(bitmapList):
    """
    Gets the maximum width and height for a bitmap list
    """
    width = 0
    height = 0
    for bmp in bitmapList:
        width = max(width, bmp.GetWidth())
        height = max(height, bmp.GetHeight())
    return (width, height)


# -*- coding: utf-8 -*-
import wx
from ctx import localcontext as context



class CreationDialog(object):
    """Class decorator for creation dialog. With this decorator
    the decorated method only needs to send the dialog arguments"""
    def __init__(self, dialogType, objectType):
        """"""
        self._dialogType = dialogType
        self._objectType = objectType

    def __call__(self, method):
        """"""
        import tran.TransactionStack as stack

        def wrapped_call(*args, **kwargs):
            """Code dialog creation"""
            dialog = self._dialogType(*method(*args, **kwargs))
            rtv = dialog.ShowModal()
            if rtv != wx.ID_OK:
                return False
            kw_or_kwlargs = dialog.get_kwargs()
            #si kw_or_kwlargs es None, no se ha de hacer nada
            if kw_or_kwlargs:
                # si kw_or_kwlargs es una lista, se interpreta que se han de crear
                # multiples objetos y se trata de una lista de kwargs
                if type(kw_or_kwlargs) is list:
                    for kwargs in kw_or_kwlargs:
                        self._objectType(**kwargs)
                else:
                    obj = self._objectType(**kw_or_kwlargs)
                    # give the possibility of insert object name in transaction label
                    if obj and stack.InTransaction() and hasattr(obj, '_name'):
                        sname = stack.instance.GetName()
                        if sname:
                            sname = sname.format(obj._name)
                            stack.instance.SetName(sname)
            return True
        return wrapped_call


class CreationDialogEx(object):
    """Class decorator for creation dialog with multilanguage support.
    The dialog and the object type for, is returned by the method"""
    def __init__(self):
        """"""
        super(CreationDialogEx, self).__init__()

    def __call__(self, method):
        """"""
        def wrapped_call(*args, **kwargs):
            """Code dialog creation"""
            args = method(*args, **kwargs)
            self._dialogType = args[0]
            self._objectType = args[1]
            args = args[2:]
            dialog = self._dialogType(*args)
            if dialog.ShowModal() != wx.ID_OK:
                return False
            kw_or_kwlargs = dialog.get_kwargs()
            #si kw_or_kwlargs es None, no se ha de hacer nada
            if kw_or_kwlargs:
                # si kw_or_kwlargs es una lista, se interpreta que se han de crear
                # multiples objetos y se trata de una lista de kwargs
                if type(kw_or_kwlargs) is list:
                    for kwargs in kw_or_kwlargs:
                        self._objectType(**kwargs)
                else:
                    obj = self._objectType(**kw_or_kwlargs)
                    # give the possibility of insert object name in transaction label
                    if obj and stack.InTransaction() and hasattr(obj, '_name'):
                        sname = stack.instance.GetName()
                        if sname:
                            sname = sname.format(obj._name)
                            stack.instance.SetName(sname)
            return True
        return wrapped_call


# -*- coding: utf-8 -*-

"""
Created on Sun Dec 15 19:22:32 2013

@author: mel
"""

import model
import os
import copy
from model.writer import writer
import app.resources as rc
import ctx
import tran
from app.utils import cached_type


class Class(model.TComponent):
    """Implements python class representation"""
    class_container = True
    context_container = True
    folder_container = True
    diagram_container = True
    inheritance_container = True
    member_container = True
    relation_container = True
    enum_container = False
    type_container = False
    import_container = True

    # visual methods
    @tran.TransactionalMethod('move python class {}')
    def drop(self, to):
        """Drops class inside project or another folder """
        target = to.inner_class_container
        if not target or self.project != target.project:
            return False  # avoid move classes between projects
        index = 0
        tran.TransactionalMoveObject(
            object=self, origin=self.parent, target=target, index=index)
        return True

    def __init__(self, **kwargs):
        """Initialization method"""
        self._deriv = kwargs.get('derivatives', [])
        self._memberPrefix = kwargs.get('prefix', '_')
        self._export = kwargs.get('export', False)

        self._lastHdrTime = None
        self._lastSrcTime = None
        super(Class, self).__init__(**kwargs)
        # add object derivative
        if not kwargs.get('raw', False):
            if self._name != 'object':
                for cls in self.project.level_classes:
                    if cls._name == 'object':
                        model.py.Inheritance(parent=self,
                            ancestor=cls, note='standard inheritance')
                        break
        k = self.inner_module or self.inner_package
        if k:
            k.ExportPythonCodeFiles()

    def get_kwargs(self):
        """Returns the kwargs needed for this object"""
        kwargs = {}
        kwargs['derivatives'] = self._deriv
        kwargs['prefix'] = self._memberPrefix
        kwargs.update(super(Class, self).get_kwargs())
        return kwargs

    def lower(self):
        """Criteria for sorting when generating code"""
        return self._inheritance_level

    def GetClassArguments(self):
        """Get the list of arguments required by the class"""
        return []

    def UpdateClassRelations(self):
        """This method gets invoked when something changes in
        class relations specification
        and ensure mainteinance of internal methods and ctors"""
        pass

    def AutoInit(self):
        """Update all init ctors"""
        pass

    def UpdateInitCalls(self):
        """Update the calls to init method"""
        pass

    def UpdateExitCalls(self):
        """Update the calls to exit method"""
        pass

    def AutoArgs(self):
        """Update all ctor arguments"""
        pass

    def updateSources(self, force=False):
        """does source generation"""
        pass

    def GetPreferredConstructor(self):
        """Obtiene el constructor por defecto o None"""
        return None

    def GetSerialConstructor(self):
        """Obtiene el constructor de serializacion o None"""
        return None

    @property
    def can_delete(self):
        """Check abot if class can be deleted"""
        return True

    def Delete(self):
        """Delete diagram objects"""
        #new version
        for dia in self.project(model.ClassDiagram):
            # Check if inherit is in
            elem = dia.FindElement(self)
            if elem is not None:
                dia.SaveState()
                dia.RemoveElement(elem)
        k = self.inner_module or self.inner_package
        super(Class, self).Delete()
        if k:
            k.ExportPythonCodeFiles()

    def RemoveRelations(self):
        """Utility for undo/redo"""
        super(Class, self).RemoveRelations()

    def RestoreRelations(self):
        """Utility for undo/redo"""
        super(Class, self).RestoreRelations()

    def SaveState(self):
        """Utility for saving state"""
        super(Class, self).SaveState()

    def OnUndoRedoRemoving(self, root=True):
        """Prepare object to delete"""
        super(Class, self).OnUndoRedoRemoving(root)

    def OnUndoRedoChanged(self):
        """Update from app"""
        super(Class, self).OnUndoRedoChanged()

    def OnUndoRedoAdd(self):
        """Restore object from undo"""
        super(Class, self).OnUndoRedoAdd()

    def ExportPythonCode(self, wf):
        """Export python code"""
        from Inheritance import Inheritance
        wf.writenl()
        ins = ','.join([x._name for x in self[Inheritance]])
        if ins:
            ins = '(' + ins + ')'
        wf.openbrace('{self.label}{ins}:'.format(self=self, ins=ins))
        wf.writecomment(self._note)
        wf.writenl()

        #filters
        class_data = lambda x: x._scope == 'class' and x.inner_class == self

        #filter for instance members
        #instace_data = lambda x: x.self._scope == 'instance'

        #filter for skipping nested classes in methods
        same_class = lambda x: x.inner_class == self

        from Import import Import
        from MemberData import MemberData
        from InitMethod import InitMethod
        from MemberMethod import MemberMethod

        wf.writeln("#imports")
        for obj in self(Import, filter=same_class, cut=True):
            obj.ExportPythonCode(wf)
        wf.writenl()

        wf.writeln("#global data")
        for obj in self(MemberData, filter=class_data, cut=False):
            obj.ExportPythonCode(wf)
        wf.writenl()

        wf.writeln("#init method")
        for obj in self(InitMethod, filter=same_class, cut=True):
            obj.ExportPythonCode(wf)
        wf.writenl()

        wf.writeln("#member methods")
        for obj in self(MemberMethod, filter=same_class, cut=True):
            obj.ExportPythonCode(wf)
        wf.writenl()

        wf.closebrace()
        wf.writenl()

        return True

    @property
    def bitmap_index(self):
        """Index of tree image"""
        return rc.GetBitmapIndex('py_class')

    @property
    def label(self):
        """Get tree label"""
        return 'class {clase._name}'.format(clase=self)

    @property
    def path_name(self):
        """Get the full path"""
        if self.inner_module is not None:
            return '{self.inner_module.path_name}.{self._name}'.format(self=self)
        return self._name

    @property
    def file_container(self):
        """get the container filename (without extension)"""
        if self.inner_module:
            return self.inner_module._name
        if self.inner_package:
            return self.inner_package._name
        return self.project._name

    @property
    def context_name(self):
        """get the class name with path through parent classes"""
        pclass = self.parent.inner_class
        if pclass:
            return '{pclass.context_name}.{self._name}'.format(pclass=pclass, self=self)
        return self._name

    @property
    def tree_label(self):
        """Get tree label"""
        cos = 'class'
        return '{cos} {self.parent.scope}{self._name}'.format(cos=cos,
            self=self)

    def IsAncestor(self, cls):
        """Checks for ancestor relationship"""
        if cls is None:
            return False
        from model.py import Inheritance
        if not Inheritance in cls._child:
            return False
        inhmap = cls[Inheritance]
        for inh in inhmap:
            if inh._ancestor == self:
                return True
            if self.IsAncestor(inh._ancestor):
                return True
        return False

    def ExistMemberData(self, name):
        """Check recursively about the existence of nested child member"""
        # TO-DO
        return False

    def UpdateInitMethod(self):
        """Update the init methods"""
        # TO-DO
        pass

    def UpdateExitMethod(self):
        """update the exit methods"""
        # TO-DO
        pass

    @property
    def inner_class(self):
        """Get the inner class container"""
        return self

    @property
    def outer_class(self):
        """Get the outer class container"""
        return (self.parent and self.parent.outer_class) or self

    @property
    def is_class_methods(self):
        """returns the list of all is_class_methods"""
        # TO-DO
        return []

    @property
    def init_methods(self):
        """returns the setup relation method"""
        # TO-DO
        return []

    @property
    def exit_methods(self):
        """returns the clean relations method"""
        # TO-DO
        return []

    @property
    def nested_classes(self):
        """Returns the list of nested classes (including self)"""
        if type(self.parent) not in [model.Folder, model.py.Class]:
            return [self]
        return self.parent.nested_classes + [self]

    @property
    def scope(self):
        """Get the scope"""
        return '{self.parent.scope}{self._name}.'.format(self=self)

    @property
    def inheritance(self):
        """Get the inheritance list"""
        # TO-DO
        return []





"""Subclass of NewVariable, which is generated by wxFormBuilder."""

import re
import wx
import wxx
from activity.models.ui import ui as ui
import app.resources as rc


# Implementing NewVariable
class PyVariableDialog(ui.NewPyVariable):
    """
    This dialog allows to setup a non-member
    variable.
    """
    @wxx.SetInfo(__doc__)
    def __init__(self, parent, container):
        """Dialog initialization"""
        super(PyVariableDialog, self).__init__(parent)
        self._container = container
        self.m_textCtrl6.SetFocus()
        icon = wx.EmptyIcon()
        icon.CopyFromBitmap(rc.GetBitmap("py_variable"))
        self.SetIcon(icon)

    def CopyAttributes(self, variable):
        """Get the atributes"""
        variable._name = self._name
        variable._value = self._value
        variable.note = self._note

    def SetAttributes(self, member):
        """Set the attributes"""
        self.m_textCtrl6.SetValue(member._name)
        self.m_textCtrl8.SetValue(member._value)
        self.m_richText1.SetValue(member._note)
        self.SetTitle("Edit variable")

    def Validate(self):
        """Dialog validation"""
        self._name = self.m_textCtrl6.GetValue()
        if len(self._name) == 0:
            wx.MessageBox("Variable name must not be empty", "Error",
                wx.OK | wx.CENTER | wx.ICON_ERROR, self)
            return False
        if re.match("^[A-Za-z_][0-9A-Za-z_]*$", self._name) is None:
            wx.MessageBox("Variable name contains invalid characters", "Error",
                wx.OK | wx.CENTER | wx.ICON_ERROR, self)
            return False
        self._value = self.m_textCtrl8.GetValue()
        self._note = self.m_richText1.GetValue()
        return True

    def get_kwargs(self):
        """return arguments for object instance"""
        return  {'parent': self._container, 'name': self._name,
            'value': self._value, 'note': self._note}

    def OnCancel(self, event):
        """cancel event handler"""
        self.EndModal(wx.ID_CANCEL)

    def OnOK(self, event):
        """ok event handler"""
        if self.Validate():
            self.EndModal(wx.ID_OK)




"""Subclass of UserSections, which is generated by wxFormBuilder."""
import wxx
from activity.models.ui import ui as ui


# Implementing UserSections
class UserSectionsDialog(ui.UserSections):
    """
    This dialog allows to edit user sections and put
    custom code inside generated code.
    """
    @wxx.SetInfo(__doc__)
    def __init__(self, parent, container):
        """initialization"""
        super(UserSectionsDialog, self).__init__(parent)
        self.container = container

    # Handlers for UserSections events.
    def OnLeaveFocus(self, event):
        event.Skip()

    def OnGetFocus(self, event):
        event.Skip()

    def CopyAttributes(self, cls):
        """Copy attributes to class"""
        cls.user_code_h1 = self.m_h1.GetText()
        cls.user_code_h2 = self.m_h2.GetText()
        cls.user_code_h3 = self.m_h3.GetText()
        cls.user_code_s1 = self.m_s1.GetText()
        cls.user_code_s2 = self.m_s2.GetText()
        cls.user_code_s3 = self.m_s3.GetText()

    def SetAttributes(self, cls):
        """Setup attributes for editing already class"""
        self.m_h1.SetText(cls.user_code_h1)
        self.m_h2.SetText(cls.user_code_h2)
        self.m_h3.SetText(cls.user_code_h3)
        self.m_s1.SetText(cls.user_code_s1)
        self.m_s2.SetText(cls.user_code_s2)
        self.m_s3.SetText(cls.user_code_s3)



"""Subclass of GenerateSources, which is generated by wxFormBuilder."""

import wx
import wxx
import app

from activity.models.ui import ui as ui
from ctx import theContext as context


# Implementing GenerateSources
class GenerateSourcesDialog(ui.GenerateSources):
    """
    This method exports the project model to language files.
    The files may be written only if they have changes or may
    be forced to be regenerated by using the checkbox
    'overwrite all'
    """
    @wxx.SetInfo(__doc__)
    def __init__(self, parent, project):
        """"Initialization"""
        super(GenerateSourcesDialog, self).__init__(parent)
        self.project = project
        self.m_author.SetValue(project._author)
        #setup report
        sz = self.m_report.GetClientSize()
        self.m_report.ClearAll()
        sz.x -= 5
        self.m_report.InsertColumn(0, "Operation", wx.LIST_FORMAT_LEFT, sz.x)

    # Handlers for GenerateSources events.
    def OnGenerate(self, event):
        """When the Generate button is pressed"""
        force = self.m_overwrite.GetValue()
        context.app.SaveProject(self.project)
        self.project.ExportCodeFiles(force, self)

    def AppendReportLine(self, line, style=wx.OK):
        """Append a information line on report"""
        self.m_report.InsertStringItem(0, line)
        self.Refresh()
        self.Update()






"""Subclass of FindText, which is generated by wxFormBuilder."""

import wx
import wxx
import app


# Implementing FindText
class FindText(app.FindText):
    """
    This dialog allows to search for text.
    Press F3 for search again for more occurrences.
    Press Shift + F3 for searching back.
    """
    @wxx.SetInfo(__doc__)
    def __init__(self, parent):
        """Initialize dialog"""
        super(FindText, self).__init__(parent)

    def Validate(self):
        """Process OnOk event"""
        self.seach_text = self.m_search_string.GetValue()
        if not len(self.seach_text):
            wx.MessageBox("Search text must be non empty", "Error",
                wx.OK | wx.CENTER | wx.ICON_ERROR, self)
            return False
        return True

    # Handlers for FindText events.
    def OnCancel(self, event):
        """Cancel seach"""
        self.EndModal(wx.ID_CANCEL)

    def OnOK(self, event):
        """Begin search"""
        if self.Validate():
            self.EndModal(wx.ID_OK)



# -*- coding: utf-8 -*-

###########################################################################
## Python code generated with wxFormBuilder (version Jun  6 2014)
## http://www.wxformbuilder.org/
##
## PLEASE DO "NOT" EDIT THIS FILE!
###########################################################################

import wx
import wx.xrc
import wx.propgrid as pg

import gettext
_ = gettext.gettext

###########################################################################
## Class AstExplorerPane
###########################################################################

class AstExplorerPane ( wx.Panel ):
	
	def __init__( self, parent ):
		wx.Panel.__init__ ( self, parent, id = wx.ID_ANY, pos = wx.DefaultPosition, size = wx.Size( 474,434 ), style = wx.TAB_TRAVERSAL )
		
		fgSizer245 = wx.FlexGridSizer( 1, 1, 0, 0 )
		fgSizer245.AddGrowableCol( 0 )
		fgSizer245.AddGrowableRow( 0 )
		fgSizer245.SetFlexibleDirection( wx.BOTH )
		fgSizer245.SetNonFlexibleGrowMode( wx.FLEX_GROWMODE_SPECIFIED )
		
		self.m_splitter3 = wx.SplitterWindow( self, wx.ID_ANY, wx.DefaultPosition, wx.DefaultSize, wx.SP_3D )
		self.m_splitter3.Bind( wx.EVT_IDLE, self.m_splitter3OnIdle )
		
		self.m_panel32 = wx.Panel( self.m_splitter3, wx.ID_ANY, wx.DefaultPosition, wx.DefaultSize, wx.TAB_TRAVERSAL )
		fgSizer250 = wx.FlexGridSizer( 1, 1, 0, 0 )
		fgSizer250.AddGrowableCol( 0 )
		fgSizer250.AddGrowableRow( 0 )
		fgSizer250.SetFlexibleDirection( wx.BOTH )
		fgSizer250.SetNonFlexibleGrowMode( wx.FLEX_GROWMODE_SPECIFIED )
		
		self.m_splitter4 = wx.SplitterWindow( self.m_panel32, wx.ID_ANY, wx.DefaultPosition, wx.DefaultSize, wx.SP_3D )
		self.m_splitter4.Bind( wx.EVT_IDLE, self.m_splitter4OnIdle )
		
		self.m_panel30 = wx.Panel( self.m_splitter4, wx.ID_ANY, wx.DefaultPosition, wx.DefaultSize, wx.TAB_TRAVERSAL )
		fgSizer246 = wx.FlexGridSizer( 1, 1, 0, 0 )
		fgSizer246.AddGrowableCol( 0 )
		fgSizer246.AddGrowableRow( 0 )
		fgSizer246.SetFlexibleDirection( wx.BOTH )
		fgSizer246.SetNonFlexibleGrowMode( wx.FLEX_GROWMODE_SPECIFIED )
		
		self.m_tree = wx.TreeCtrl( self.m_panel30, wx.ID_ANY, wx.DefaultPosition, wx.DefaultSize, wx.TR_DEFAULT_STYLE|wx.TR_HIDE_ROOT|wx.TR_SINGLE )
		fgSizer246.Add( self.m_tree, 0, wx.EXPAND|wx.TOP|wx.LEFT, 5 )
		
		
		self.m_panel30.SetSizer( fgSizer246 )
		self.m_panel30.Layout()
		fgSizer246.Fit( self.m_panel30 )
		self.m_panel31 = wx.Panel( self.m_splitter4, wx.ID_ANY, wx.DefaultPosition, wx.DefaultSize, wx.TAB_TRAVERSAL )
		fgSizer247 = wx.FlexGridSizer( 1, 1, 0, 0 )
		fgSizer247.AddGrowableCol( 0 )
		fgSizer247.AddGrowableRow( 0 )
		fgSizer247.SetFlexibleDirection( wx.BOTH )
		fgSizer247.SetNonFlexibleGrowMode( wx.FLEX_GROWMODE_SPECIFIED )
		
		self.m_propertyGrid = pg.PropertyGrid(self.m_panel31, wx.ID_ANY, wx.DefaultPosition, wx.DefaultSize, wx.propgrid.PG_DEFAULT_STYLE)
		fgSizer247.Add( self.m_propertyGrid, 0, wx.EXPAND|wx.BOTTOM|wx.LEFT, 5 )
		
		
		self.m_panel31.SetSizer( fgSizer247 )
		self.m_panel31.Layout()
		fgSizer247.Fit( self.m_panel31 )
		self.m_splitter4.SplitHorizontally( self.m_panel30, self.m_panel31, 317 )
		fgSizer250.Add( self.m_splitter4, 1, wx.EXPAND, 5 )
		
		
		self.m_panel32.SetSizer( fgSizer250 )
		self.m_panel32.Layout()
		fgSizer250.Fit( self.m_panel32 )
		self.m_splitter3.Initialize( self.m_panel32 )
		fgSizer245.Add( self.m_splitter3, 1, wx.EXPAND, 5 )
		
		
		self.SetSizer( fgSizer245 )
		self.Layout()
		
		# Connect Events
		self.m_tree.Bind( wx.EVT_TREE_SEL_CHANGED, self.OnSelectItem )
	
	def __del__( self ):
		pass
	
	
	# Virtual event handlers, overide them in your derived class
	def OnSelectItem( self, event ):
		event.Skip()
	
	def m_splitter3OnIdle( self, event ):
		self.m_splitter3.SetSashPosition( 0 )
		self.m_splitter3.Unbind( wx.EVT_IDLE )
	
	def m_splitter4OnIdle( self, event ):
		self.m_splitter4.SetSashPosition( 317 )
		self.m_splitter4.Unbind( wx.EVT_IDLE )
	

###########################################################################
## Class ChooseFile
###########################################################################

class ChooseFile ( wx.Dialog ):
	
	def __init__( self, parent ):
		wx.Dialog.__init__ ( self, parent, id = wx.ID_ANY, title = _(u"Select file"), pos = wx.DefaultPosition, size = wx.Size( 366,231 ), style = wx.DEFAULT_DIALOG_STYLE )
		
		self.SetSizeHintsSz( wx.DefaultSize, wx.DefaultSize )
		
		fgSizer242 = wx.FlexGridSizer( 2, 1, 0, 0 )
		fgSizer242.AddGrowableCol( 0 )
		fgSizer242.AddGrowableRow( 0 )
		fgSizer242.SetFlexibleDirection( wx.BOTH )
		fgSizer242.SetNonFlexibleGrowMode( wx.FLEX_GROWMODE_SPECIFIED )
		
		fgSizer244 = wx.FlexGridSizer( 1, 2, 0, 0 )
		fgSizer244.AddGrowableCol( 1 )
		fgSizer244.AddGrowableRow( 0 )
		fgSizer244.SetFlexibleDirection( wx.BOTH )
		fgSizer244.SetNonFlexibleGrowMode( wx.FLEX_GROWMODE_SPECIFIED )
		
		self.m_staticText112 = wx.StaticText( self, wx.ID_ANY, _(u"select file"), wx.DefaultPosition, wx.DefaultSize, 0 )
		self.m_staticText112.Wrap( -1 )
		fgSizer244.Add( self.m_staticText112, 0, wx.ALL|wx.ALIGN_CENTER_VERTICAL|wx.ALIGN_RIGHT, 5 )
		
		self.m_filePicker1 = wx.FilePickerCtrl( self, wx.ID_ANY, wx.EmptyString, _(u"Select a python file"), u"*.py", wx.DefaultPosition, wx.DefaultSize, wx.FLP_DEFAULT_STYLE|wx.FLP_USE_TEXTCTRL )
		fgSizer244.Add( self.m_filePicker1, 0, wx.ALL|wx.ALIGN_CENTER_VERTICAL, 5 )
		
		
		fgSizer242.Add( fgSizer244, 1, wx.ALIGN_CENTER_HORIZONTAL|wx.ALIGN_CENTER_VERTICAL, 5 )
		
		fgSizer243 = wx.FlexGridSizer( 1, 2, 0, 0 )
		fgSizer243.AddGrowableCol( 1 )
		fgSizer243.SetFlexibleDirection( wx.BOTH )
		fgSizer243.SetNonFlexibleGrowMode( wx.FLEX_GROWMODE_SPECIFIED )
		
		self.m_info = wx.BitmapButton( self, wx.ID_ANY, wx.ArtProvider.GetBitmap( wx.ART_TIP, wx.ART_BUTTON ), wx.DefaultPosition, wx.DefaultSize, wx.BU_AUTODRAW )
		fgSizer243.Add( self.m_info, 0, wx.ALL, 5 )
		
		m_sdbSizer25 = wx.StdDialogButtonSizer()
		self.m_sdbSizer25OK = wx.Button( self, wx.ID_OK )
		m_sdbSizer25.AddButton( self.m_sdbSizer25OK )
		self.m_sdbSizer25Cancel = wx.Button( self, wx.ID_CANCEL )
		m_sdbSizer25.AddButton( self.m_sdbSizer25Cancel )
		m_sdbSizer25.Realize();
		
		fgSizer243.Add( m_sdbSizer25, 1, wx.EXPAND, 5 )
		
		
		fgSizer242.Add( fgSizer243, 1, wx.EXPAND, 5 )
		
		
		self.SetSizer( fgSizer242 )
		self.Layout()
		
		self.Centre( wx.BOTH )
		
		# Connect Events
		self.m_sdbSizer25OK.Bind( wx.EVT_BUTTON, self.OnOK )
	
	def __del__( self ):
		pass
	
	
	# Virtual event handlers, overide them in your derived class
	def OnOK( self, event ):
		event.Skip()
	




# -*- coding: utf-8 -*-
import re
import mmap
import contextlib as _ctx


class scan(object):
    """
    Defines the base class for a lexical scanner.
    The scanner is composed by five logical sections:

        definitions: A list of pattern definitions that may be
        applied to rules patterns.

        lexicals: A list of output values for mached rule output.

        states: A list of scanner states.

        rules: A list of rules that may be applied secuentially
        consuming the input stream.

        methods: Set of methods applied during the work. They
        must be implemented in derived class.

    Notes:

        The scanner may be applied to file for analysis. For
        that, we must use scan.filescan(file)

        lexical values are for use into parser generator. The
        None symbol is allways a lexical value

    Adding definitions:

        We add a new definition (remember the use of raw strings)
        using the pdef method:

            pdef('alpha','[a-zA-Z]')

    Adding lexicals:

        We add lexicals for parser generator by calling plex:

            plex('identifier')

    Adding states:

        The states are inclusive or exclusive and declared with
        pstate('name',inclusive=True). As usual, the BEGIN state
        inits the scanner process and the scanner state may select
        the active rules. If the user don't defines the BEGIN
        state, it will be defined as inclusive.

    Adding rules:

        Rules are declared with the method prule specifying the
        rule name, the regexpr to match, a collection of
        start conditions (if any) and a optional method to execute
        when the rule is matched. The optional method can return
        a lexical for use in the parser generator.
        The rules follow the syntax of python regexpr module,
        but they can include single quoted definitions (and yes,
        the quotes may be now scaped for normal use). For example,

        prule('label',"'alpha'*",do=self.method)

    Conecting with parser generator:
        Pass the parser argument for conection in __init__.

    """

    def __init__(self, parser=None):
        """Initialize a lexical scanner."""
        self._defs = {}
        self._lex = [None]
        self._states = [[], []]
        self._rules = []
        self._state = None
        self._inclusive = True
        self._parser = parser
        super(scan, self).__init__()

    def pdef(self, key, value):
        """Add new definition to a lexical scanner"""
        self._defs[key] = value

    def plex(self, id):
        """Add new lexical identifier"""
        self._lex.append(id)

    def pstate(self, name, inclusive=True):
        """Add new state to scanner"""
        if inclusive:
            self._states[0].append(name)
        else:
            self._states[1].append(name)

    def prule(self, label, rule, *args, **kwargs):
        """Add a new rule to scanner"""
        s = rule
        #expand definitions
        for d in self._defs:
            s = re.sub(r"(?<!\\)'{}'".format(d),
                self._defs[d], s)
        #remove scaped quotes
        s = re.sub(r"\\'", "'", s)
        #register mask
        self._rules.append({
            'name': label,
            'pattern': re.compile(s),
            'conditions': args,
            'do': kwargs.get('do', lambda x: None)
            })

    def begin(self, state):
        """do a new state"""
        if state in self._states[0]:
            self._inclusive = True
        elif state in self._states[1]:
            self._inclusive = False
        else:
            raise ValueError('unknown state {}'.format(state))
        self._state = state

    def filescan(self, rfile):
        """do a lexical scan over a file"""
        with _ctx.closing(mmap.mmap(rfile, prot=mmap.PROT_READ)) as m:
            return scan(m[:])

    def scan(self, s):
        """do a scan. Return true if the scanner
        has matched the string, and false otherwise"""
        if None not in self._states[0] and None not in self._states[1]:
            self._state[0].append(None)
        self.begin(None)
        if self._parser:
            self._parser.reset()
        # Ok we find the next match
        pos = 0
        nxt = True
        while nxt and pos < len(s):
            nxt = False
            for rule in filter(self.active, self._rules):
                match = rule['pattern'].match(s, pos)
                if match is None:
                    continue
                value = rule['do'](match)
                if value and self._parser:
                    self._parser.put(value)
                pos += len(match.m.group(0))
                nxt = True
                break
        return nxt

    def active(self, rule):
        """Returns info about if some rule is active or not"""
        cond = rule['conditions']
        return (not len(cond) and self._inclusive) or (
                self._state in cond)





"""Subclass of EditDecorators, which is generated by wxFormBuilder."""

import wx
import wxx
from activity.models.ui import ui as ui
import app.resources as rc
import model
from PyDecorator import PyDecoratorDialog


# Implementing EditDecorators
class EditDecoratorsDialog(ui.EditDecorators):
    """
    This dialog allows to edit the decorators applied
    to some method
    """
    @wxx.SetInfo(__doc__)
    def __init__(self, parent, container):
        """Initialize dialog"""
        super(EditDecoratorsDialog, self).__init__(parent)
        icon = wx.EmptyIcon()
        icon.CopyFromBitmap(rc.GetBitmap("decorator"))
        self.SetIcon(icon)
        self._container = container
        sz = self.m_listCtrl6.GetClientSize()
        self.m_listCtrl6.InsertColumn(0, "decorators", wx.LIST_FORMAT_LEFT, sz.x - 48)
        # iterate through the decorators and insert them
        self._count = 0
        self._map = {}
        for deco in container[model.py.Decorator]:
            self.m_listCtrl6.InsertStringItem(self._count, deco._name)
            self.m_listCtrl6.SetItemData(self._count, self._count)
            self._map[self._count] = deco
            self._count = self._count + 1

    def Validate(self):
        """Validate dialog"""
        # Transform al decorators in new decorators
        for k in self._map:
            v = self._map[k]
            if type(v) is not tuple:
                name = v._name
                call = v._call
                self._map[k] = (name, call)
        return True

    # Handlers for EditDecorators events.
    def OnSelectDecorator(self, event):
        """Handles the selection of a decorator in the report"""
        self.m_bpButton65.Enable(True)
        self.m_bpButton66.Enable(True)
        n = self.m_listCtrl6.GetFirstSelected()
        if n > 0:
            self.m_bpButton67.Enable(True)
        if n + 1 < self.m_listCtrl6.GetItemCount():
            self.m_bpButton68.Enable(True)

    def OnUnselectDecorator(self, event):
        """Handles decorator unselection"""
        self.m_bpButton65.Enable(False)
        self.m_bpButton66.Enable(False)
        self.m_bpButton67.Enable(False)
        self.m_bpButton68.Enable(False)

    def OnDecoratorUp(self, event):
        """Handles the addition of a new decorator"""
        n = self.m_listCtrl6.GetFirstSelected()
        if n == wx.NOT_FOUND:
            return
        k = self.m_listCtrl6.GetItemData(n)
        name = self.m_listCtrl6.GetItemText(n)
        self.m_listCtrl6.DeleteItem(n)
        n = n - 1
        self.m_listCtrl6.InsertStringItem(n, name)
        self.m_listCtrl6.SetItemData(n, k)
        self.m_listCtrl6.Select(n)

    def OnDecoratorDown(self, event):
        """Handles the addition of a new decorator"""
        n = self.m_listCtrl6.GetFirstSelected()
        if n == wx.NOT_FOUND:
            return
        n = self.m_listCtrl6.GetFirstSelected()
        if n == wx.NOT_FOUND:
            return
        k = self.m_listCtrl6.GetItemData(n)
        name = self.m_listCtrl6.GetItemText(n)
        self.m_listCtrl6.DeleteItem(n)
        n = n + 1
        self.m_listCtrl6.InsertStringItem(n, name)
        self.m_listCtrl6.SetItemData(n, k)
        self.m_listCtrl6.Select(n)

    def OnAddDecorator(self, event):
        """Handles the addition of a new decorator"""
        dialog = PyDecoratorDialog(self)
        if dialog.ShowModal() == wx.ID_OK:
            # se anhade el nuevo decorador
            n = self.m_listCtrl6.GetItemCount()
            name = dialog._name
            call = dialog._call
            self.m_listCtrl6.InsertStringItem(n, name)
            self.m_listCtrl6.SetItemData(n, self._count)
            self._map[self._count] = (name, call)
            self.m_listCtrl6.Select(self._count)
            self._count = self._count + 1

    def OnEditDecorator(self, event):
        """Handles the edition of a decorator"""
        # get selected index
        n = self.m_listCtrl6.GetFirstSelected()
        if n == wx.NOT_FOUND:
            return
        # get key
        k = self.m_listCtrl6.GetItemData(n)
        # get object
        obj = self._map[k]
        # edit decorator
        dialog = PyDecoratorDialog(self, self._container)
        dialog.SetAttributes(obj)
        if dialog.ShowModal() == wx.ID_OK:
            name = dialog._name
            call = dialog._call
            # update the element
            self.m_listCtrl6.SetItemText(n, name)
            # replace entry
            self._map[k] = (name, call)

    def OnDeleteDecorator(self, event):
        """Handles the removal of a decorator"""
        # TODO: Implement OnDeleteDecorator
        n = self.m_listCtrl6.GetFirstSelected()
        if n == wx.NOT_FOUND:
            return
        # get key
        k = self.m_listCtrl6.GetItemData(n)
        # simply remove element and index
        self.m_listCtrl6.DeleteItem(n)
        del self._map[k]
        self._count = self._count - 1

    def OnCancel(self, event):
        """Cancels the dialog"""
        self.EndModal(wx.ID_CANCEL)

    def OnOK(self, event):
        """Validates and accepts the dialog"""
        if self.Validate():
            self.EndModal(wx.ID_OK)

    def get_kwargs(self):
        """Create the new decorators"""
        #remove previous decorators
        olds = self._container(model.py.Decorator)
        for old in olds:
            old.Delete()
        #create news
        kwargs = {'parent': self._container}
        for k in self._map:
            v = self._map[k]
            kwargs['name'] = v[0]
            kwargs['call'] = v[1]
            model.py.Decorator(**kwargs)
        return None





from TCommon import TCommon
from TComponent import TComponent
from Folder import Folder
from Library import Library
from LibrariesFolder import LibrariesFolder
from Member import Member
from Project import Project
from Workspace import Workspace
from Note import Note

from ClassDiagram import NO_WHERE
from ClassDiagram import IN_CLASS
from ClassDiagram import IN_INHERIT
from ClassDiagram import IN_AGGREG
from ClassDiagram import IN_NOTE
from ClassDiagram import IN_LINE
from ClassDiagram import IN_SHAPE
from ClassDiagram import IN_LINE_MARK

from ClassDiagram import DiagramElement
from ClassDiagram import ClosedShape
from ClassDiagram import RectShape
from ClassDiagram import ClassElement
from ClassDiagram import ConnectorSegment
from ClassDiagram import ConectorElement
from ClassDiagram import InheritanceElement
from ClassDiagram import NoteElement
from ClassDiagram import ClassDiagram

import cc
import py
import file


# -*- coding: utf-8 -*-

from ast_explorer import ast_explorer
from log_explorer import log_explorer
from web_browser import web_browser
from check_updates import check_updates

ALL = [ast_explorer, log_explorer, web_browser, check_updates]


"""Subclass of FilePane, which is generated by wxFormBuilder."""

import wx
import wx.stc
from ctx import theContext as context
import app
import tran
from app.ui.pane import BasePane
from activity.files.ui import ui as ui
from activity.models.ui import ui as amui

CLOSE_PANE_ID = wx.ID_HIGHEST + 100


# Implementing FilePane
class FilePane(BasePane, ui.FilePane):
    """Implements code editor"""
    def __init__(self, parent, mainframe, fileobj):
        """Intialization of method editor"""
        super(FilePane, self).__init__(parent)
        self._parent = parent
        self._mainframe = mainframe
        self._object = fileobj
        self._types = dict([(x._name, x) for x in fileobj.project.types])
        self._defaulBkColor = self.GetBackgroundColour()
        self.m_editor.SetText(fileobj.GetText())
        self.m_editor.HandleBreakpoints(fileobj.breakpoints)
        self.m_editor.HandleBookmarks(fileobj.bookmarks)
        self.m_editor.EnableBreakpoints(True)
        self.m_editor.EnableBookmarks(True)
        self.m_editor.Initialize(self._types)
        self.m_editor.SetReadOnly(fileobj._readOnly)
        self.Layout()
        wx.stc.EVT_STC_CHANGE(self.m_editor, self.m_editor.GetId(),
            self.OnEditorChange)
        frame = context.frame
        self.Bind(wx.EVT_MENU, handler=frame.CloseCurrentDocPane, id=CLOSE_PANE_ID)
        self.Bind(wx.EVT_MENU, handler=self.OnDelete, id=frame.delete.GetId())
        self.Bind(wx.EVT_MENU, handler=self.OnSave, id=frame.saveProject.GetId())
        self.Bind(wx.EVT_UPDATE_UI, handler=self.OnUpdateSave, id=frame.saveProject.GetId())
        aTable = wx.AcceleratorTable([
            wx.AcceleratorEntry(wx.ACCEL_NORMAL, wx.WXK_DELETE, frame.delete.GetId()),
            wx.AcceleratorEntry(wx.ACCEL_ALT | wx.ACCEL_CTRL, ord('F'), amui.ID_TOGGLE_FULLSCREEN),
            wx.AcceleratorEntry(wx.ACCEL_CTRL, ord('W'), CLOSE_PANE_ID)
        ])
        self.SetAcceleratorTable(aTable)

    def Refresh(self):
        """Update editor from external changes (like undo/redo)"""
        oldText = self.m_editor.GetText()
        newText = self._object.GetText()
        if oldText == newText:
            return
        line = self.m_editor.GetFirstVisibleLine()
        pos = self.m_editor.GetCurrentPos()
        self.m_editor.Freeze()
        self.m_editor.SetText(self._object.GetText())
        self.m_editor.SetModified(False)
        try:
            self.m_editor.SetFirstVisibleLine(line)
            self.m_editor.SetCurrentPos(pos)
        except:
            pass
        self.m_editor.Thaw()

    def OnEditorChange(self, event):
        """Called when editor status change"""
        if self.m_editor.GetModify():
            self.SetBackgroundColour(wx.LIGHT_GREY)
        else:
            self.SetBackgroundColour(self._defaulBkColor)

    def GotoLine(self, line, select=False):
        """Called for goto to line"""
        self.m_editor.GotoLine(line, select)

    @tran.TransactionalMethod('modify file {}')
    def Commit(self):
        """Do commit changes"""
        if self.m_editor.GetModify():
            self._object.SaveState()
            self._object.SetText(self.m_editor.GetText())
            if wx.__version__ >= '3.0.0':
                self.m_editor.SetModified(False)
            self.SetBackgroundColour(self._defaulBkColor)
            return True
        else:
            return False

    @property
    def breakpoint(self):
        """return info about if the current line holds breakpoint"""
        return self.m_editor.breakpoint

    def ToggleBreakpoint(self, event):
        """toggle breakpoint"""
        self.m_editor.ToggleBreakpoint(event)

    @property
    def line(self):
        """return current line"""
        return self.m_editor.GetCurrentLine()

    def OnDelete(self, event):
        """Handle delete key"""
        (b, e) = self.m_editor.GetSelection()
        if b == e:
            self.m_editor.CharRight()
        self.m_editor.DeleteBack()

    def OnUpdateSave(self, event):
        """Handles update event"""
        event.Enable(self.m_editor.GetModify())

    def OnSave(self, event):
        """Handles save event"""
        self.Commit()





# -*- coding: utf-8 -*-
"""This class defines a class decorator useful for creating methods that handles transactionality
in automatical manner"""

from tran import TransactionStack


class TransactionalMethod(object):
    """Class decorator for transactional methods"""
    def __init__(self, transactionName=''):
        """Function decorator for transactional methods.
        The transactionName is the name the transaction receives"""
        self._transactionName = transactionName

    def __call__(self, method):
        """"""
        def wrapped_call(*args, **kwargs):
            """Code for transamodeltional method"""
            # If we are inside an open transaction, we dont open
            # another again. Instead, do the associated operations
            # and return his 'something changes?' return value
            if TransactionStack.InTransaction():
                return method(*args, **kwargs)
            # Ok, we on top of new transaction. We do our work,
            # commit changes or rollback when no changes or exception happens
            import traceback
            import sys
            try:
                TransactionStack.DoBeginTransaction(self._transactionName)
                value = method(*args, **kwargs)
                if value:
                    TransactionStack.DoCommit()
                else:
                    TransactionStack.DoRollback()
                return value
            except Exception as inst:
                traceback.print_exc(file=sys.stdout)
                print type(inst)     # the exception instance
                print inst.args      # arguments stored in .args
                print inst
                TransactionStack.DoRollback()
                raise inst
        return wrapped_call



# -*- coding: utf-8 -*-

# -*- coding: utf-8 -*-

"""
This class allows to set some common handlers and methods for
easy view declaration"""

import wx
import wxx
import tran
import app
import model
from ctx import localcontext as context


class BasePane(object):
    """Declares a base pane class"""

    def __init__(self, *args, **kwargs):
        """Initialize base view"""
        self._evtHandler = wx.PyEvtHandler()
        self._handler_installed = False
        self._focus = False
        super(BasePane, self).__init__(*args, **kwargs)

    def BindSpecial(self, evtID, handler, source=None, id=-1, id2=-1):
        """Do a special binding for pluggable event handler and also internal event handler"""
        self._evtHandler.Bind(evtID, handler, source=source, id=id, id2=id2)
        self.Bind(evtID, handler, source=source, id=id, id2=id2)

    def install_handler(self):
        """Installs the event handler"""
        if self._handler_installed:
            return False  # already installed
        context.frame.PushEventHandler(self._evtHandler)
        self._handler_installed = True
        return True  # ok, installed

    def remove_handler(self):
        """Remove the event handler"""
        if not self._handler_installed:
            return False  # already removed
        # pop event handlers until found this
        other_event_handlers = []
        evth = context.frame.PopEventHandler()
        try:
            while evth != self._evtHandler:
                other_event_handlers.append(evth)
                evth = context.frame.PopEventHandler()
            if evth == self._evtHandler:
                self._handler_installed = False
            result = True
        except:
            print "Fatal error : missing event handler"
            self._handler_installed = False
            result = False
        # reinstall the event handlers chain
        while len(other_event_handlers):
            context.frame.PushEventHandler(other_event_handlers.pop())
        return result

    @property
    def is_event_handler(self):
        """return info about if the handler is installed"""
        return self._handler_installed

    def PreDelete(self):
        """Remove the event handler"""
        self.remove_handler()

    def NotifyShow(self):
        """Called where the git view is show"""
        self.install_handler()

    def NotifyHide(self):
        """Called where the git view is hidden"""
        self.remove_handler()





# -*- coding: utf-8 -*-

# -*- coding: utf-8 -*-
"""
Created on Sun Dec 22 22:08:46 2013

@author: mel
"""
from MemberMethod import MemberMethod


class ExitMethod(MemberMethod):
    """Implements is_class method"""
    context_container = True
    argument_container = True

    def __init__(self, **kwargs):
        """Initialization"""
        kwargs['name'] = '__exit__'
        kwargs['readonly'] = True
        super(ExitMethod, self).__init__(**kwargs)

    def get_kwargs(self):
        """Returns the kwargs needed for this object"""
        kwargs = {}
        kwargs.update(super(ExitMethod, self).get_kwargs())
        return kwargs

    def OnUndoRedoChanged(self):
        """Update from app"""
        super(ExitMethod, self).OnUndoRedoChanged()

    def OnUndoRedoRemoving(self, root=True):
        """Prepare for delete"""
        super(ExitMethod, self).OnUndoRedoRemoving(root)

    def OnUndoRedoAdd(self):
        """Restore object from undo"""
        super(ExitMethod, self).OnUndoRedoAdd()



"""Subclass of NewInheritance, which is generated by wxFormBuilder."""

import wx
import wxx
import model
from activity.models.ui import ui as ui
import app.resources as rc


# Implementing NewPynheritance
class PyInheritanceDialog(ui.NewPyInheritance):
    """
    This dialog allows to setup inheritance from
    another struct or class.
    """
    @wxx.SetInfo(__doc__)
    def __init__(self, parent, container):
        """Initialize dialog"""
        super(PyInheritanceDialog, self).__init__(parent)
        self._name = ""
        self._original = ""
        self._element = container
        self.project = container.project
        self.__initClassBoxes__()
        self.m_choice11.SetFocus()
        icon = wx.EmptyIcon()
        icon.CopyFromBitmap(rc.GetBitmap("py_inheritance"))
        self.SetIcon(icon)

    def __initClassBoxes__(self):
        """Initialize class boxes"""
        classes = self.project.level_classes
        for clase in classes:
            if self._element == clase:
                continue
            if self._element.IsAncestor(clase):
                continue
            self.m_choice11.Append(clase._name, clase)
        if len(classes) > 0:
            self.m_choice11.SetSelection(0)

    def Validate(self):
        """Validation"""
        self._name = self.m_choice11.GetValue()
        iSel = self.m_choice11.FindString(self._name)
        if iSel == wx.NOT_FOUND:
            self._ancestor = None
        else:
            self._ancestor = self.m_choice11.GetClientData(iSel)
        self._note = self.m_richText3.GetValue()
        if self._name != self._original:
            inheritance = self._element(model.cc.Inheritance,
                filter=lambda x: x.inner_class == self._element, cut=True)
            if self._name in [x._name for x in inheritance]:
                wx.MessageBox("Inheritance from " + self._name +
                " already exists", "Error",
                    wx.OK | wx.CENTER | wx.ICON_ERROR, self)
                return False
        return True

    def CopyAttributes(self, inheritance):
        """Copy attributes to inheritance"""
        inheritance._name = self._name
        inheritance.SetAncestor(self._ancestor)
        inheritance.note = self._note

    def SetAttributes(self, inheritance):
        """Setup attributes for editing already inheritance"""
        self._original = inheritance._name
        self.m_choice11.SetValue(inheritance._name)
        #self.m_choice11.SetSelection(iSel)
        if inheritance.note:
            self.m_richText3.SetValue(inheritance.note)
        self.SetTitle("Edit inheritance")

    def get_kwargs(self):
        """returns arguments suitable for object instance"""
        return {'parent': self._element,
            'ancestor': self._ancestor,
            'name': self._name,
            'note': self._note}

    # Handlers for NewMethod events.
    def OnCancel(self, event):
        """cancel event handler"""
        self.EndModal(wx.ID_CANCEL)

    # Handlers for NewInheritance events.
    def OnOK(self, event):
        """Process OnOk event"""
        """ok event handler"""
        if self.Validate():
            self.EndModal(wx.ID_OK)



"""Subclass of NewRelation, which is generated by wxFormBuilder."""

import re
import wx
import wxx
from activity.models.ui import ui as ui
import app.resources as rc
import model.cc


# Implementing NewRelation
class RelationDialog(ui.NewRelation):
    """
    This dialog allows to setup a relation between classes
    and/or  instances.
    It's possible to set the cardinalities in the relation, both
    for the parent as for the child, and also the implementation
    method.
    The process of code generation is controlled by xml
    editable plugins and may be customized and expanded.
    """
    @wxx.SetInfo(__doc__)
    def __init__(self, parent, container, to=None):
        """Initialize dialog"""
        super(RelationDialog, self).__init__(parent)
        self._element = container
        self._to = to
        self.project = container.project
        self._hasMinCardFrom = False
        self._hasMaxCardFrom = False
        self._hasMinCardTo = False
        self._hasMaxCardTo = False
        self._minCardFrom = 0
        self._maxCardFrom = -1  # means infinity
        self._minCardTo = 0
        self._maxCardTo = -1  # means infinity
        self.__initClassBoxes__()
        self.m_choice4.SetFocus()
        icon = wx.EmptyIcon()
        icon.CopyFromBitmap(rc.GetBitmap("relation"))
        self.SetIcon(icon)

    def __initClassBoxes__(self):
        """Initialize class boxes"""
        nameFrom = ""
        nameTo = ""
        if type(self._element) is model.cc.Class:
            nameFrom = self._element._name
        if type(self._to) is model.cc.Class:
            nameTo = self._to._name
        classes = self.project(model.cc.Class)
        index1 = wx.NOT_FOUND
        index2 = wx.NOT_FOUND
        for clase in classes:
            i1 = self.m_choice4.Append(clase._name, clase)
            i2 = self.m_choice41.Append(clase._name, clase)
            if clase._name == nameFrom:
                index1 = i1
            if clase._name == nameTo:
                index2 = i2
        if len(nameFrom):
            nameFrom = nameFrom[0].upper() + nameFrom[1:]
        if len(nameTo):
            nameTo = nameTo[0].upper() + nameTo[1:]
        if not index1 == wx.NOT_FOUND:
            self.m_choice4.SetSelection(index1)
            self.m_textCtrl10.SetValue("From" + nameFrom)
        if not index2 == wx.NOT_FOUND:
            self.m_choice41.SetSelection(index2)
            self.m_textCtrl101.SetValue("To" + nameTo)

    def OnChangeFrom(self, event):
        """When changed the from"""
        index = self.m_choice4.GetCurrentSelection()
        name = self.m_choice4.GetString(index)
        name = name[0].upper() + name[1:]
        self.m_textCtrl10.SetValue("From" + name)

    def OnMinCardinalFrom(self, event):
        """Enables or disables from limits"""
        if self.m_chkMinCardFrom.GetValue():
            self.m_minFrom.Enable(True)
            self.m_minFrom.SetValue('1')
        else:
            self.m_minFrom.Enable(False)
        event.Skip()

    def OnMaxCardinalFrom(self, event):
        """Enables or disables from limits"""
        if self.m_chkMaxCardFrom.GetValue():
            self.m_maxFrom.Enable(True)
            self.m_maxFrom.SetValue('infinity')
        else:
            self.m_maxFrom.Enable(False)
        event.Skip()

    def OnMinCardinalTo(self, event):
        """Enables or disables to limits"""
        if self.m_chkMinCardTo.GetValue():
            self.m_minTo.Enable(True)
            self.m_minTo.SetValue('1')
        else:
            self.m_minTo.Enable(False)
        event.Skip()

    def OnMaxCardinalTo(self, event):
        """Enables or disables to limits"""
        if self.m_chkMaxCardTo.GetValue():
            self.m_maxTo.Enable(True)
            self.m_maxTo.SetValue('infinity')
        else:
            self.m_maxTo.Enable(False)
        event.Skip()

    def OnChangeTo(self, event):
        """When changed the to"""
        index = self.m_choice41.GetCurrentSelection()
        name = self.m_choice41.GetString(index)
        name = name[0].upper() + name[1:]
        self.m_textCtrl101.SetValue("To" + name)

    def Validate(self):
        """Validates dialog correctess"""
        self._fromName = self.m_textCtrl10.GetValue()
        self._toName = self.m_textCtrl101.GetValue()
        if re.match("^[A-Za-z_][0-9A-Za-z_]*$", self._fromName) is None:
            wx.MessageBox("From alias contains invalid characters", "Error",
                wx.OK | wx.CENTER | wx.ICON_ERROR, self)
            return False
        if re.match("^[A-Za-z_][0-9A-Za-z_]*$", self._toName) is None:
            wx.MessageBox("To alias contains invalid characters", "Error",
                wx.OK | wx.CENTER | wx.ICON_ERROR, self)
            return False
        index = self.m_choice4.GetCurrentSelection()
        if index is wx.NOT_FOUND:
            return False
        self._from = self.m_choice4.GetClientData(index)
        index = self.m_choice41.GetCurrentSelection()
        if index is wx.NOT_FOUND:
            return False
        self._to = self.m_choice41.GetClientData(index)
        self._critical = self.m_checkBox23.GetValue()
        self._global = self.m_checkBox47.GetValue()
        self._filter = self.m_checkBox24.GetValue()
        self._unique = self.m_checkBox25.GetValue()
        self._unikey = None
        if self.m_chkMinCardFrom.GetValue():
            self._fromMinCardinal = self.m_minFrom.GetValue()
        else:
            self._fromMinCardinal = None
        if self.m_chkMaxCardFrom.GetValue():
            self._fromMaxCardinal = self.m_maxFrom.GetValue()
        else:
            self._fromMaxCardinal = None
        if self.m_chkMinCardTo.GetValue():
            self._toMinCardinal = self.m_minTo.GetValue()
        else:
            self._toMinCardinal = None
        if self.m_chkMaxCardTo.GetValue():
            self._toMaxCardinal = self.m_maxTo.GetValue()
        else:
            self._toMaxCardinal = None
        self._fromAmodeless = self.m_choice12.GetStringSelection()
        self._toAmodeless = self.m_choice121.GetStringSelection()
        self._implementation = self.m_choice19.GetStringSelection()
        self._note = self.m_richText3.GetValue()
        return True

    def get_kwargs(self):
        """returns arguments suitable for instance"""
        return {'fromName': self._fromName, 'toName': self._toName,
            'FROM': self._from, 'TO': self._to, 'critical': self._critical,
            'Global': self._global, 'filter': self._filter, 'unique': self._unique,
            'unikey': self._unikey, 'frommin': self._fromMinCardinal,
            'fommax': self._fromMaxCardinal, 'tomin': self._toMinCardinal,
            'tomax': self._toMaxCardinal, 'fromaccess': self._fromAmodeless,
            'toaccess': self._toAmodeless, 'implementation': self._implementation,
            'note': self._note}

    def OnCancel(self, event):
        """cancel event handler"""
        self.EndModal(wx.ID_CANCEL)

    def CopyAttributes(self, relation):
        """Transfer data to relation items"""
        # as the dialog can be invoked from one side of the relation, we first
        # must access to the relation reference
        if hasattr(relation, '_key'):
            relation = relation._key
        relation._critical = self.m_checkBox23.GetValue()
        relation._global = self.m_checkBox47.GetValue()
        relation._filter = self.m_checkBox24.GetValue()
        relation._unique = self.m_checkBox25.GetValue()
        relation._unikey = None
        relation._implementation = self.m_choice19.GetStringSelection()
        if self.m_chkMinCardFrom.GetValue():
            relation._FROM._minCardinal = self.m_minFrom.GetValue()
        else:
            relation._FROM._minCardinal = None
        if self.m_chkMaxCardFrom.GetValue():
            relation._FROM._maxCardinal = self.m_maxFrom.GetValue()
        else:
            relation._FROM._maxCardinal = None
        if self.m_chkMinCardTo.GetValue():
            relation._TO._minCardinal = self.m_minTo.GetValue()
        else:
            relation._TO._minCardinal = None
        if self.m_chkMaxCardTo.GetValue():
            relation._TO._maxCardinal = self.m_maxTo.GetValue()
        else:
            relation._TO._maxCardinal = None
        relation._FROM._access = self.m_choice12.GetStringSelection()
        relation._TO._access = self.m_choice121.GetStringSelection()
        relation._FROM._name = self.m_textCtrl10.GetValue()
        relation._TO._name = self.m_textCtrl101.GetValue()
        relation.note = self.m_richText3.GetValue()

    def SetAttributes(self, relation):
        """Copy attributes from relation object"""
        # as the dialog can be invoked from one side of the relation, we first
        # must access to the relation reference
        if hasattr(relation, '_key'):
            relation = relation._key
        self.m_checkBox23.SetValue(relation._critical)
        self.m_checkBox47.SetValue(relation._global)
        self.m_checkBox24.SetValue(relation._filter)
        self.m_checkBox25.SetValue(relation._unique)
        if relation._FROM._minCardinal is not None:
            self.m_chkMinCardFrom.SetValue(True)
            self.m_minFrom.Enable()
            self.m_minFrom.SetValue(relation._FROM._minCardinal)
        else:
            self.m_chkMinCardFrom.SetValue(False)
            self.m_minFrom.Enable(False)
            self.m_minFrom.SetValue('0')
        if relation._FROM._maxCardinal is not None:
            self.m_chkMaxCardFrom.SetValue(True)
            self.m_maxFrom.Enable()
            self.m_maxFrom.SetValue(relation._FROM._maxCardinal)
        else:
            self.m_chkMaxCardFrom.SetValue(False)
            self.m_maxFrom.Enable(False)
            self.m_maxFrom.SetValue('infinity')
        if relation._TO._minCardinal is not None:
            self.m_chkMinCardTo.SetValue(True)
            self.m_minTo.Enable()
            self.m_minTo.SetValue(relation._TO._minCardinal)
        else:
            self.m_chkMinCardTo.SetValue(False)
            self.m_minTo.Enable(False)
            self.m_minTo.SetValue('0')
        if relation._TO._maxCardinal is not None:
            self.m_chkMaxCardTo.SetValue(True)
            self.m_maxTo.Enable()
            self.m_maxTo.SetValue(relation._TO._maxCardinal)
        else:
            self.m_chkMaxCardTo.SetValue(False)
            self.m_maxTo.Enable(False)
            self.m_maxTo.SetValue('infinity')
        index = self.m_choice19.FindString(relation._implementation)
        self.m_choice19.SetSelection(index)
        self.m_richText3.SetValue(relation.note)
        index = self.m_choice4.FindString(relation._TO.parent._name)
        self.m_choice4.SetSelection(index)
        index = self.m_choice41.FindString(relation._FROM.parent._name)
        self.m_choice41.SetSelection(index)
        #self.OnChangeFrom(None)
        #self.OnChangeTo(None)
        # once created, relationship cannot be moved this way
        self.m_choice4.Enable(False)
        self.m_choice41.Enable(False)
        self.m_choice12.SetStringSelection(relation._FROM._access)
        self.m_choice121.SetStringSelection(relation._TO._access)
        self.m_textCtrl10.SetValue(relation._FROM._name)
        self.m_textCtrl101.SetValue(relation._TO._name)
        self.SetTitle("Edit relation")

    def OnOK(self, event):
        """Implements OnOK"""
        if self.Validate():
            self.EndModal(wx.ID_OK)



# -*- coding: utf-8 -*-

"""
    Introducing delayed methods
    ===========================

    A delayed method is a smart method that detects in what environment is invoked
    and avoid duplicated and expensive calls.

    In transactional-free context, a delayed method behaves like normal a method:
    do his work and returns.

    Meanwhile, in the context of any open transaction or undo/redo operation, a
    delayed method enqueues the call, avoiding duplicates for the end of the
    transactional operation.

    That results in reduction of redundant calls done, for example while deleting
    a big set of elements at once.

    Creating a delayed method with a decorator is elegant and simple: nothing
    changes from the user point of view.

    The main target of delayed methods is obviously graphical updates, but surprisingly
    the origin of this idea was to minimice code regeneration.

"""


from tran import TransactionStack


def DelayedMethod(fifo=True):
    """Delayad with precedence"""
    def decoration(method):
        """Function decorator for delayed methods."""
        def wrapped_call(*args, **kwargs):
            """Code for delayed method"""
            # If we are not in transactional context, simply do the call
            if not TransactionStack.InTransaction() and not TransactionStack.InUndoRedo():
                # do inmmediate call
                return method(*args, **kwargs)
            # Skip explicit filtered calls
            if method in TransactionStack.delayedCallsFiltered:
                return
            # Ok, check the method inside the transaction
            entry = TransactionStack.delayedCalls.get(method, [])
            if (args, kwargs) in entry:
                return
            if entry:
                if fifo:
                    TransactionStack.delayedCalls[method].append((args, kwargs))
                else:
                    TransactionStack.delayedCalls[method].insert(0, (args, kwargs))
            else:
                TransactionStack.delayedCalls[method] = [(args, kwargs)]
        wrapped_call.inner = method
        return wrapped_call
    return decoration



# -*- coding: utf-8 -*-
from model.writer import writer


class filewriter(writer):
    """filewiter is a comodity class for writing C/C++ files"""
    def __init__(self, fhandle, language="c++"):
        """initialization"""
        super(filewriter, self).__init__(language)
        self.file = fhandle

    def writeln(self, txt=''):
        """write a line or secuence of lines with ident"""
        for l in txt.splitlines():
            self.file.write("{0}{1}\n".format(self.cident, l))

    def writenl(self, count=1):
        """write a newline"""
        for i in range(0, count):
            self.file.write("\n")





"""Subclass of VirtualMethods, which is generated by wxFormBuilder."""

import wx
import wxx
from activity.models.ui import ui as ui
import model


# Implementing VirtualMethods
class VirtualMethodsDialog(ui.VirtualMethods):
    """
    This dialog allows to add inherited virtual methods.
    """
    @wxx.SetInfo(__doc__)
    def __init__(self, parent, container):
        "initialization"
        super(VirtualMethodsDialog, self).__init__(parent)
        # container es la clase destino
        self.container = container
        self.parent_class = container.inner_class
        # create a map of feasible casts
        self._classes = []
        self._methods = []
        for k in self.parent_class.inheritance:
            self.visit(k._ancestor)
        # scan the available virtual methods
        virtual = lambda x: x._virtual
        for cls in self._classes:
            members = lambda x: x.inner_class == cls
            self._methods.extend(filter(virtual,
                cls(model.cc.MemberMethod, members, True)))
        # create a map from method names to implementations
        self._map = {}
        members = lambda x: x.inner_class == self.parent_class
        for k in self._methods:
            s = k.label.replace('=0', '')
            override = lambda x: x._virtual and x.label == s
            if s not in self._map:
                #find the method inside the class
                self._map[s] = (k,
                    filter(override, self.parent_class(model.cc.MemberMethod,
                        members, True)))
        # ok, do insertion remebering implementations
        pos = 0
        for k in self._map:
            v = self._map[k]
            self.m_checkList2.Insert(k, pos, v)
            if v[1]:
                self.m_checkList2.Check(pos)
            pos = pos + 1
        aTable = wx.AcceleratorTable([
            wx.AcceleratorEntry(wx.ACCEL_NORMAL, wx.WXK_RETURN, wx.ID_OK),
        #tree navigation facilities
        ])
        self.SetAcceleratorTable(aTable)

    def visit(self, k):
        """Add ancestor branch"""
        for l in k.inheritance:
            self.visit(l._ancestor)
        self._classes.append(k)

    def get_kwargs(self):
        """Returns kwargs dictionary suitable for objects creation"""
        kwargs_list = []
        for item in range(0, self.m_checkList2.GetCount()):
            v = self.m_checkList2.GetClientData(item)
            c = self.m_checkList2.IsChecked(item)
            if  (c and v[1]) or (not c and not v[1]):
                continue
            if c:
                kwargs = {}
                kwargs = v[0].get_kwargs()
                kwargs['parent'] = self.container
                kwargs['pure'] = False
                kwargs['declare'] = True
                kwargs['implement'] = True
                kwargs_list.append(kwargs)
            else:
                for m in v[1]:
                    m.Delete()
        return kwargs_list



# -*- coding: utf-8 -*-

import wx
import app

from _workspace import _workspace
from _folder import _folder
from _folder_open import _folder_open
from _project import _project
from _class import _class
from _py_class import _py_class
from _friend import _friend
from _method import _method
from _py_method import _py_method
from _classdiagram import _classdiagram
from _member import _member
from _py_member import _py_member
from _constructor import _constructor
from _py_init import _py_init
from _context import _context
from _destructor import _destructor
from _inheritance import _inheritance
from _py_inheritance import _py_inheritance
from _relation import _relation
from _folderT import _folderT
from _folderP import _folderP
from _folderI import _folderI
from _type import _type
from _parent import _parent
from _library import _library
from _child import _child
from _cppfile import _cppfile
from _pyfile import _pyfile
from _hfile import _hfile
from _mfile import _mfile
from _python import _python
from _protected import _protected
from _private import _private
from _target import _target
from _cppproject import _cppproject
from _namespace import _namespace
from _module import _module
from _py_module import _py_module
from _function import _function
from _py_function import _py_function
from _data import _data
from _py_variable import _py_variable
from _enum import _enum
from _tres import _tres
from _py_args import _py_args
from _py_kwargs import _py_kwargs
from _py_import import _py_import
from _models import _models
from _files import _files
from _tasks import _tasks
from _git import _git
from _py_argument import _py_argument
from _py_package import _py_package
from _start import _start
from _info import _info
from _mini_logo import _mini_logo
from _git_repo import _git_repo
from _git_file import _git_file
from _git_file_modified import _git_file_modified
from _git_file_staged import _git_file_staged
from _git_file_deleted import _git_file_deleted
from _file import _file
from _reload import _reload
from _decorator import _decorator
from _bookmark import _bookmark
from _git_remote import _git_remote
from _glass_clock import _glass_clock
from _folder_pendings import _folder_pendings
from _folder_pendings_open import _folder_pendings_open
from _folder_current import _folder_current
from _folder_current_open import _folder_current_open
from _refresh import _refresh
from _run import _run
from _run_file import _run_file
from _stop import _stop
from _debug import _debug
from _debug_file import _debug_file
from _step_into import _step_into
from _step_out import _step_out
from _step_over import _step_over
from _continue import _continue
from _pyboost import _pyboost

#from _class_small import _class_small


_xpm = [
    _class, _constructor, _member, _method, _destructor,
    _inheritance, _type, _parent, _child, _enum, _py_argument,
    _py_args, _py_kwargs, _py_init, _py_member, _py_method,
    _py_inheritance, _classdiagram, _relation, _folderT,
    _cppfile, _hfile, _mfile, _target, _cppproject, _namespace,
    _folder, _folder_open, _project, _module, _function,
    _py_function, _data, _py_variable, _tres, _friend, _folderP,
    _library, _python, _workspace, _context, _py_module,
    _py_package, _py_class, _models, _files, _tasks, _py_import,
    _pyfile, _start, _info, _mini_logo, _git, _git_repo,
    _git_file, _git_file_modified, _git_file_staged,
    _git_file_deleted, _file, _reload, _decorator, _folderI,
    _git_remote, _glass_clock, _folder_pendings, _folder_pendings_open,
    _folder_current, _folder_current_open, _refresh, _run,
    _run_file, _stop, _step_into, _step_out, _step_over, _continue,
    _debug, _debug_file, _pyboost]


_accessed = [_class, _constructor, _member, _method, _destructor, _inheritance,
    _type, _parent, _child, _enum]
_access = {'protected': _protected, 'private': _private}

_bitmapList = []
_bitmapAmodeless = {}

#def ImageIndex(name):
#    """Finds the xpm by name and returns the index (or wx.NOT_FOUND)"""
#    try:
#        return _xpm.index(eval('_{}'.format(name)))
#    except:
#        return wx.NOT_FOUND


def CreateBitmapList():
    """Crea la lista de bitmaps"""
    global _bitmapList
    global _xpm
    global _bitmapAmodeless
    global _accessed
    bmpWithAmodeless = []
    for xpm in _xpm:
        bmp = wx.BitmapFromXPMData(xpm)
        _bitmapList.append(bmp)
        if xpm in _accessed:
            bmpWithAmodeless.append(bmp)
    _bitmapAmodeless['public'] = {}
    for bmp in bmpWithAmodeless:
        _bitmapAmodeless['public'][_bitmapList.index(bmp)] = _bitmapList.index(bmp)
    for access in _access:
        prefixBmp = wx.BitmapFromXPMData(_access[access])
        _bitmapAmodeless[access] = app.SuffixBitmapList(prefixBmp,
            bmpWithAmodeless, _bitmapList)


#def BitmapIndex(name, access=None):
#    """Finds the xpm by name and returns the index (or wx.NOT_FOUND)"""
#    try:
#        return _xpm.index(eval('_{}'.format(name)))
#    except:
#        return wx.NOT_FOUND

#def GetBitmapName(name, access=None):
#    """Return the bitmap index by name"""
#    return GetBitmapIndex(ImageIndex(name), access)

def GetBitmapIndex(name, access=None):
    """Return the bitmap index"""
    try:
        base_index = _xpm.index(eval('_{}'.format(name)))
    except:
        return wx.NOT_FOUND
    if access is None:
        return base_index
    global _bitmapList
    if len(_bitmapList) == 0:
        CreateBitmapList()
    return _bitmapAmodeless[access][base_index]


def GetBitmap(name, access=None):
    """Get the specific bitmap"""
    index = GetBitmapIndex(name, access)
    if index == wx.NOT_FOUND:
        raise KeyError('bitmap {} not found'.format(name))
        return None
    return _bitmapList[index]


def GetBitmapImageList():
    """Returns the bitmap image list"""
    global _bitmapList
    if len(_bitmapList) == 0:
        CreateBitmapList()
    (w, h) = app.GetBitmapMaxSize(_bitmapList)
    if wx.__version__ >= '3.0.0.0':
        w = 0
    imglist = wx.ImageList(w, h, True, len(_bitmapList))
    for bmp in _bitmapList:
        imglist.Add(bmp)
    return imglist




# -*- coding: utf-8 -*-

import model
import app.resources as rc


class ImportsFolder(model.Folder):
    """Implements a Folder representation"""
    class_container = False
    type_container = False
    function_container = False
    variable_container = False
    module_container = False
    namespace_container = False
    function_container = False
    member_container = False
    import_container = True

    def __init__(self, **kwargs):
        """Initialization"""
        kwargs['name'] = 'imports'
        super(ImportsFolder, self).__init__(**kwargs)
        self.update_container()

    def update_container(self):
        """Update the container info"""
        self.context_container = False
        self.folder_container = False
        self.diagram_container = False
        self.namespace_container = False

    def ExportPythonCode(self, wf):
        """Write code"""
        from Import import Import
        for x in self[Import]:
            x.ExportPythonCode(wf)

    @property
    def bitmap_index(self):
        """Index of tree image"""
        return rc.GetBitmapIndex("folderI")



"""Subclass of Locals, which is generated by wxFormBuilder."""
import wx
import wx.propgrid as pg
from activity.files.ui import ui as ui


# Implementing Locals
class Locals(ui.Locals):
    """Pane showing current local vars"""

    def __init__(self, parent, handler):
        """Initialize pane"""
        self._handler = handler
        super(Locals, self).__init__(parent)

    # Update data
    def UpdateData(self):
        """Update the threads info"""
        self.Freeze()
        self.m_locals.Clear()
        _locals = self._handler.locals
        # _locals is a dictionary. We have here some things to consider
        # a) since dictionnary are not an ordered set, if we simply insert
        #    values, we get erratic order between steps... we need to
        #    sort keys before
        keys = _locals.keys()
        keys.sort()
        for name in keys:
            value = _locals[name]
            self.AddProperty(name, value)
        self.Thaw()

    def AddProperty(self, name, value=wx.EmptyString, append_in=None):
        """Add a new property"""
        #print 'property:{name} = {value}'.format(name=name, value=value)
        t = type(value)
        if t in [dict, list, tuple]:
            prop = pg.NewStringProperty(name, name, '<composed>')
        else:
            try:
                svalue = str(value)
                prop = pg.NewStringProperty(name, name, svalue)
            except:
                print 'failed', value
                svalue = '<undisplayable>'
                prop = pg.NewStringProperty(name, name, svalue)
        if append_in:
            self.m_locals.AppendIn(append_in, prop)
        else:
            self.m_locals.Append(prop)
        self.m_locals.SetPropertyReadOnly(prop)
        if t is dict:
            keys = [str(x) for x in value]
            keys.sort()
            trans = dict([(str(x), x) for x in value])  # key translator
            for k in keys:
                self.AddProperty(k, value[trans[k]], prop)
            self.m_locals.Collapse(prop)
        elif t in [list, tuple]:
            values = list(value)
            values.sort()
            for k in values:
                try:
                    sk = str(k)
                    self.AddProperty(sk, wx.EmptyString, prop)
                except:
                    print 'failed', k
                    pass
            self.m_locals.Collapse(prop)





"""Subclass of ContextItems, which is generated by wxFormBuilder."""

import copy
import wx
import wxx

from activity.models.ui import ui as ui
import model.decorator
import app.resources as rc


# Implementing ContextItems
class ContextItems(ui.ContextItems):
    """
    Dialog for editing context definitions. Context elements have two functions.

        - May act as macros controlled by the 'enabled' checkbox:

            We can control the inclusion of the definition inserted in the definition box.

        - May act as macro context labeling:

            We can prefix and sufix both the element declaration and the element definition
            with the code inserted in the corresponding boxes.

    These functionalities can be conbined by defining macros used on the context labeling.
    """
    @wxx.SetInfo(__doc__)
    def __init__(self, parent, project):
        """Initialize dialog"""
        super(ContextItems, self).__init__(parent)
        if not hasattr(project, '_contexts'):
            project._contexts = []
        self._contexts = copy.deepcopy(project._contexts)
        count = 0
        for item in self._contexts:
            self.m_listBox1.Insert(item._name, count, item)
        self._selected = None
        self._selectedIndex = None
        icon = wx.EmptyIcon()
        icon.CopyFromBitmap(rc.GetBitmap("context"))
        self.SetIcon(icon)


    def OnSelectItem(self, event):
        """Handles context item selection"""
        obj = self.m_listBox1.GetClientData(event.GetSelection())
        self._selectedIndex = event.GetSelection()
        self._selected = obj
        # replace al info in the editors
        self.m_textCtrl40.SetValue(obj.name)
        self.m_checkBox64.SetValue(obj._enable)
        self.m_richText15.SetValue(obj._define)
        self.m_richText11.SetValue(obj._prefix_declaration)
        self.m_richText13.SetValue(obj._sufix_declaration)
        self.m_richText12.SetValue(obj._prefix_implementation)
        self.m_richText14.SetValue(obj._sufix_implementation)
        self.m_richText1.SetValue(obj.note)
        self.m_button31.Enable(True)

    def OnChangeName(self, event):
        """Handle change context name event"""
        txt = self.m_textCtrl40.GetValue()
        if self._selected is not None:
            self._selected._name = txt
        if self._selectedIndex is not None:
            self.m_listBox1.SetString(self._selectedIndex, txt)
        event.Skip()

    def OnEnabledChanged(self, event):
        """Handle on enable change"""
        if self._selected is not None:
            self._selected._enable = self.m_checkBox64.GetValue()
        event.Skip()

    def OnChangeDefinition(self, event):
        """Handle change context name event"""
        txt = self.m_richText15.GetValue()
        if self._selected is not None:
            self._selected._define = txt
        event.Skip()

    def OnChangeDeclarationPrefix(self, event):
        """Handle change context declaration prefix event"""
        txt = self.m_richText11.GetValue()
        if self._selected is not None:
            self._selected._prefix_declaration = txt
        event.Skip()

    def OnChangeDeclarationSufix(self, event):
        """Handle change context declaration sufix event"""
        txt = self.m_richText13.GetValue()
        if self._selected is not None:
            self._selected._sufix_declaration = txt
        event.Skip()

    def OnChangeImplementationPrefix(self, event):
        """Handle change context implementation prefix event"""
        txt = self.m_richText12.GetValue()
        if self._selected is not None:
            self._selected._prefix_implementation = txt
        event.Skip()

    def OnChangeImplementationSufix(self, event):
        """Handle change context implementation sufix event"""
        txt = self.m_richText14.GetValue()
        if self._selected is not None:
            self._selected._sufix_implementation = txt
        event.Skip()

    def OnChangeNote(self, event):
        """Handle change context name event"""
        txt = self.m_richText1.GetValue()
        if self._selected is not None:
            self._selected.note = txt
        event.Skip()

    def OnAddNewContext(self, event):
        """Handle add new context button event"""
        count = self.m_listBox1.GetCount()
        name = self.m_textCtrl40.GetValue()
        if self._selected is not None or len(name) == 0:
            name = 'new context'
            self._selected = None
            self._selectedIndex = None
            self.m_textCtrl40.SetValue(name)
            self.m_checkBox64.SetValue(True)
            self.m_richText11.SetValue('')
            self.m_richText12.SetValue('')
            self.m_richText13.SetValue('')
            self.m_richText14.SetValue('')
            self.m_richText15.SetValue('')
            self.m_richText1.SetValue('')
        item = model.decorator.ContextItem(
            name=name,
            define=self.m_richText15.GetValue(),
            enable=self.m_checkBox64.GetValue(),
            prefix_declaration=self.m_richText11.GetValue(),
            sufix_declaration=self.m_richText13.GetValue(),
            prefix_implementation=self.m_richText12.GetValue(),
            sufix_implementation=self.m_richText14.GetValue(),
            note=self.m_richText1.GetValue()
            )
        self._contexts.append(item)
        self.m_listBox1.Insert(item._name, count, item)
        self._selected = item
        self._selectedIndex = count
        self.m_listBox1.Select(self._selectedIndex)
        self.m_button31.Enable(False)

    def OnRemoveContext(self, event):
        """Handle remove context"""
        self._contexts.remove(self._selected)
        self.m_listBox1.Delete(self._selectedIndex)
        self._selected = None
        self._selectedIndex = None
        self.m_textCtrl40.SetValue('')
        self.m_checkBox64.SetValue(True)
        self.m_richText11.SetValue('')
        self.m_richText12.SetValue('')
        self.m_richText13.SetValue('')
        self.m_richText14.SetValue('')
        self.m_richText15.SetValue('')
        self.m_richText1.SetValue('')
        self.m_button31.Enable(False)

    def Validate(self):
        """Checks validation"""
        return True

    # Handlers for ContextItems events.
    def OnOk(self, event):
        """Implements Ok"""
        if self.Validate():
            self.EndModal(wx.ID_OK)

    def CopyAttributes(self, project):
        """Transfer attributes to project"""
        project._contexts = self._contexts




"""Subclass of NewNote, which is generated by wxFormBuilder."""

import wx
import wxx

from activity.models.ui import ui as ui


# Implementing NewNote
class NoteDialog(ui.NewNote):
    """
    This is the note dialog, that allows to setup
    a class diagram note.
    """
    @wxx.SetInfo(__doc__)
    def __init__(self, parent, container=None):
        """Initialize dialog"""
        super(NoteDialog, self).__init__(parent)

    # Handlers for NoteDialog events.

    def Validate(self):
        """Check dialog"""
        self._note = self.m_text.GetValue()
        return True

    def CopyAttributes(self, note):
        """Transfer attributes to folder"""
        note.note = self._note

    def SetAttributes(self, note):
        """Transfer attributes from existing constructor"""
        self._note = note.note
        self.m_text.SetValue(self._note)
        self.SetTitle("Edit note")

    def get_kwargs(self):
        """return arguments suitable for object instance"""
        return {'note': self._note}

    def OnOK(self, event):
        """Handles Ok button in dialog"""
        if self.Validate():
            self.EndModal(wx.ID_OK)
